@@ -2,19 +2,20 @@
 
 
 import logging
-from collections import OrderedDict
+import re
 
 from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, linegraph
 
 # Initialise the logger
 log = logging.getLogger(__name__)
 
+VERSION_REGEX = r"AdapterRemoval ver. ([\d\.]+)"
+
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="Adapter Removal",
@@ -47,7 +48,7 @@ class MultiqcModule(BaseMultiqcModule):
             self.s_name = f["s_name"]
             try:
                 parsed_data = self.parse_settings_file(f)
-            except UserWarning:
+            except ModuleNoSamplesFound:
                 continue
             if parsed_data is not None:
                 self.adapter_removal_data[self.s_name] = parsed_data
@@ -57,7 +58,7 @@ class MultiqcModule(BaseMultiqcModule):
         self.adapter_removal_data = self.ignore_samples(self.adapter_removal_data)
 
         if len(self.adapter_removal_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.adapter_removal_data)))
 
@@ -71,7 +72,6 @@ class MultiqcModule(BaseMultiqcModule):
         self.adapter_removal_length_dist_plot()
 
     def parse_settings_file(self, f):
-
         self.result_data = {
             "total": None,
             "unaligned": None,
@@ -85,11 +85,15 @@ class MultiqcModule(BaseMultiqcModule):
 
         block_title = None
         for i, line in enumerate(f["f"]):
-
             line = line.rstrip("\n")
             if line == "":
                 continue
 
+            if line.startswith("AdapterRemoval"):
+                version_match = re.search(VERSION_REGEX, line)
+                if version_match:
+                    self.add_software_version(version_match.group(1), self.s_name)
+
             if not block_title:
                 block_title = "header"
                 settings_data[block_title].append(str(line))
@@ -129,7 +133,7 @@ class MultiqcModule(BaseMultiqcModule):
         # biological/technical relevance is not clear -> skip
         if self.__read_type == "single" and self.__collapsed:
             log.warning("Case single-end and collapse is not " "implemented -> File %s skipped" % self.s_name)
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
     def set_trim_stat(self, trim_data):
         required = [
@@ -218,7 +222,6 @@ class MultiqcModule(BaseMultiqcModule):
             self.result_data["percent_discarded"] = 0
 
     def set_len_dist(self, len_dist_data):
-
         for line in len_dist_data[1:]:
             l_data = line.rstrip("\n").split("\t")
             l_data = list(map(int, l_data))
@@ -264,24 +267,24 @@ class MultiqcModule(BaseMultiqcModule):
                     self.len_dist_plot_data["all"][self.s_name][l_data[0]] = l_data[7]
 
     def adapter_removal_stats_table(self):
-
-        headers = OrderedDict()
-        headers["percent_aligned"] = {
-            "title": "% Trimmed",
-            "description": "% trimmed reads",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "RdYlGn-rev",
-            "shared_key": "percent_aligned",
-        }
-        headers["aligned_total"] = {
-            "title": "{} Reads Trimmed".format(config.read_count_prefix),
-            "description": "Total trimmed reads ({})".format(config.read_count_desc),
-            "modify": lambda x: x * config.read_count_multiplier,
-            "min": 0,
-            "scale": "PuBu",
-            "shared_key": "read_count",
+        headers = {
+            "percent_aligned": {
+                "title": "% Trimmed",
+                "description": "% trimmed reads",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "RdYlGn-rev",
+                "shared_key": "percent_aligned",
+            },
+            "aligned_total": {
+                "title": "{} Reads Trimmed".format(config.read_count_prefix),
+                "description": "Total trimmed reads ({})".format(config.read_count_desc),
+                "modify": lambda x: x * config.read_count_multiplier,
+                "min": 0,
+                "scale": "PuBu",
+                "shared_key": "read_count",
+            },
         }
         if self.__any_collapsed:
             headers["percent_collapsed"] = {
@@ -305,7 +308,6 @@ class MultiqcModule(BaseMultiqcModule):
         self.general_stats_addcols(self.adapter_removal_data, headers)
 
     def adapter_removal_retained_chart(self):
-
         pconfig = {
             "title": "Adapter Removal: Discarded Reads",
             "id": "ar_retained_plot",
@@ -314,8 +316,7 @@ class MultiqcModule(BaseMultiqcModule):
             "cpswitch_counts_label": "Number of Reads",
         }
 
-        cats_pec = OrderedDict()
-
+        cats_pec = {}
         if self.__any_paired:
             cats_pec["paired_reads"] = {"name": "Uncollapsed Paired Reads"}
 
@@ -344,7 +345,6 @@ class MultiqcModule(BaseMultiqcModule):
         )
 
     def adapter_removal_length_dist_plot(self):
-
         pconfig = {
             "title": "Adapter Removal: Length Distribution",
             "id": "ar_length_count_plot",
