@@ -1,12 +1,9 @@
-#!/usr/bin/env python
-
 """ MultiQC module to parse output from biobambam2 """
 
-from __future__ import print_function
-from collections import OrderedDict
+
 import logging
 
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.modules.picard import MarkDuplicates
 
 # Initialise the logger
@@ -28,31 +25,24 @@ class MultiqcModule(BaseMultiqcModule):
         )
 
         # Set up class objects to hold parsed data
-        self.general_stats_headers = OrderedDict()
+        self.general_stats_headers = dict()
         self.general_stats_data = dict()
         n = dict()
 
-        n["bamsormadup"] = MarkDuplicates.parse_reports(
-            self,
-            log_key="biobambam2/bamsormadup",
-            section_name="bamsormadup",
-            section_anchor="biobambam2-bamsormadup",
-            plot_title="biobambam2: bamsormadup deduplication stats",
-            plot_id="biobambam2_bamsormadup_plot",
-            data_filename="bamsormadup_bamsormadup",
-        )
+        n["bamsormadup"] = MarkDuplicates.parse_reports(self, "bamsormadup")
         if n["bamsormadup"] > 0:
             log.info("Found {} bamsormadup reports".format(n["bamsormadup"]))
 
         # Exit if we didn't find anything
         if sum(n.values()) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         # Add to the General Stats table (has to be called once per MultiQC module)
         self.general_stats_addcols(self.general_stats_data, self.general_stats_headers)
 
     # Helper functions
-    def multiply_hundred(self, val):
+    @staticmethod
+    def multiply_hundred(val):
         try:
             val = float(val) * 100
         except ValueError:
