@@ -4,9 +4,8 @@
 import logging
 import os
 import re
-from collections import OrderedDict
 
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph
 
 # Initialise the logger
@@ -17,7 +16,6 @@ class MultiqcModule(BaseMultiqcModule):
     """ClipAndMerge module"""
 
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="ClipAndMerge",
@@ -36,7 +34,7 @@ class MultiqcModule(BaseMultiqcModule):
         self.clipandmerge_data = self.ignore_samples(self.clipandmerge_data)
 
         if len(self.clipandmerge_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.clipandmerge_data)))
 
@@ -59,12 +57,17 @@ class MultiqcModule(BaseMultiqcModule):
             "usable_forward_no_pairing_reverse": r"Number of usable forward reads with no pairing reverse read:\s+(\d+)",
             "usable_reverse_no_pairing_forward": r"Number of usable reverse reads with no pairing forward read:\s+(\d+)",
             "identifier": r"SampleID:\s+(\S+)",
+            "version": r"ClipAndMerge \(v. ([\d\.]+)\)",
         }
 
         parsed_data = dict()
         for k, r in regexes.items():
             r_search = re.search(r, f["f"], re.MULTILINE)
             if r_search:
+                if k == "versions":
+                    parsed_data[k] = r_search.group(1)
+                    continue
+
                 try:
                     parsed_data[k] = float(r_search.group(1))
                 except ValueError:
@@ -75,21 +78,26 @@ class MultiqcModule(BaseMultiqcModule):
             if "identifier" in parsed_data:
                 s_name = self.clean_s_name(parsed_data["identifier"], f)
             self.clipandmerge_data[s_name] = parsed_data
+
+            if "version" in parsed_data:
+                self.add_software_version(parsed_data["version"], s_name)
+
             self.add_data_source(f)
 
     def clipandmerge_general_stats_table(self):
         """Take the parsed stats from the ClipAndMerge report and add it to the
         basic stats table at the top of the report"""
 
-        headers = OrderedDict()
-        headers["percentage"] = {
-            "title": "% Merged",
-            "description": "Percentage of reads merged",
-            "min": 0,
-            "max": 100,
-            "suffix": "%",
-            "scale": "Greens",
-            "format": "{:,.2f}",
+        headers = {
+            "percentage": {
+                "title": "% Merged",
+                "description": "Percentage of reads merged",
+                "min": 0,
+                "max": 100,
+                "suffix": "%",
+                "scale": "Greens",
+                "format": "{:,.2f}",
+            }
         }
         self.general_stats_addcols(self.clipandmerge_data, headers)
 
@@ -97,12 +105,13 @@ class MultiqcModule(BaseMultiqcModule):
         """Make the HighCharts HTML to plot the duplication rates"""
 
         # Specify the order of the different possible categories
-        keys = OrderedDict()
-        keys["merged_reads"] = {"name": "Merged Reads"}
-        keys["usable_not_merged_forward"] = {"name": "Usable, not merged (forward)"}
-        keys["usable_not_merged_reverse"] = {"name": "Usable, not merged (reverse)"}
-        keys["usable_forward_no_pairing_reverse"] = {"name": "Usable forward-only"}
-        keys["usable_reverse_no_pairing_forward"] = {"name": "Usable reverse-only"}
+        keys = {
+            "merged_reads": {"name": "Merged Reads"},
+            "usable_not_merged_forward": {"name": "Usable, not merged (forward)"},
+            "usable_not_merged_reverse": {"name": "Usable, not merged (reverse)"},
+            "usable_forward_no_pairing_reverse": {"name": "Usable forward-only"},
+            "usable_reverse_no_pairing_forward": {"name": "Usable reverse-only"},
+        }
 
         # Config for the plot
         config = {
