@@ -1,9 +1,7 @@
-#!/usr/bin/env python
-
 """ MultiQC submodule to parse output from deepTools plotFingerprint """
 
 import logging
-from collections import OrderedDict
+
 import numpy as np
 
 from multiqc.plots import linegraph
@@ -12,7 +10,7 @@ from multiqc.plots import linegraph
 log = logging.getLogger(__name__)
 
 
-class plotFingerprintMixin:
+class PlotFingerprintMixin:
     def parse_plotFingerprint(self):
         """Find plotFingerprint output. Both --outQualityMetrics and --outRawCounts"""
         self.deeptools_plotFingerprintOutQualityMetrics = dict()
@@ -38,6 +36,10 @@ class plotFingerprintMixin:
             if len(parsed_data) > 0:
                 self.add_data_source(f, section="plotFingerprint")
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         self.deeptools_plotFingerprintOutRawCounts = self.ignore_samples(self.deeptools_plotFingerprintOutRawCounts)
         self.deeptools_plotFingerprintOutQualityMetrics = self.ignore_samples(
             self.deeptools_plotFingerprintOutQualityMetrics
@@ -115,7 +117,7 @@ class plotFingerprintMixin:
             s_name = self.clean_s_name(cols[0], f)
             if s_name in d:
                 log.warning("Replacing duplicate sample {}.".format(s_name))
-            d[s_name] = OrderedDict()
+            d[s_name] = dict()
 
             try:
                 for i, c in enumerate(cols[1:]):
@@ -129,7 +131,7 @@ class plotFingerprintMixin:
                     if header[i] == "AUC" or header[i] == "Synthetic AUC":
                         continue
                     d[s_name][header[i]] = float(c)
-            except:
+            except:  # noqa: E722
                 log.warning(
                     "{} was initially flagged as the output from plotFingerprint --outQualityMetrics, but that seems to not be the case. Skipping...".format(
                         f["fn"]
