@@ -1,9 +1,8 @@
 """ MultiQC module to parse output from ngs-disambiguate. """
 
 import logging
-from collections import OrderedDict
 
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph
 
 log = logging.getLogger(__name__)
@@ -40,15 +39,20 @@ class MultiqcModule(BaseMultiqcModule):
         self.data = self.ignore_samples(self.data)
 
         if len(self.data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found %d reports", len(self.data))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         self.add_stats_table()
         self.add_stats_plot()
         self.write_stats_to_file()
 
-    def parse_summary(self, contents):
+    @staticmethod
+    def parse_summary(contents):
         """Parses summary file into a dictionary of counts."""
 
         lines = contents.strip().split("\n")
@@ -92,10 +96,11 @@ class MultiqcModule(BaseMultiqcModule):
     def add_stats_plot(self):
         """Plots alignment stats as bargraph."""
 
-        keys = OrderedDict()
-        keys["species_a"] = {"color": "#437bb1", "name": "Species a"}
-        keys["species_b"] = {"color": "#b1084c", "name": "Species b"}
-        keys["ambiguous"] = {"color": "#333333", "name": "Ambiguous"}
+        keys = {
+            "species_a": {"color": "#437bb1", "name": "Species a"},
+            "species_b": {"color": "#b1084c", "name": "Species b"},
+            "ambiguous": {"color": "#333333", "name": "Ambiguous"},
+        }
 
         plot_config = {
             "id": "disambiguated_alignments",
