@@ -3,10 +3,11 @@
 
 import json
 import logging
-from collections import OrderedDict
+import re
+from typing import Dict, Optional, Tuple
 
 from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, linegraph
 
 # Initialise the logger
@@ -19,7 +20,6 @@ class MultiqcModule(BaseMultiqcModule):
     """
 
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="fastp",
@@ -29,6 +29,21 @@ class MultiqcModule(BaseMultiqcModule):
             doi="10.1093/bioinformatics/bty560",
         )
 
+        data_by_sample = dict()
+        for f in self.find_log_files("fastp", filehandles=True):
+            s_name, parsed_json = self.parse_fastp_log(f)
+            if not s_name:
+                continue
+            if s_name in data_by_sample:
+                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
+            data_by_sample[s_name] = parsed_json
+
+        # Filter to strip out ignored sample names
+        data_by_sample = self.ignore_samples(data_by_sample)
+        if len(data_by_sample) == 0:
+            raise ModuleNoSamplesFound
+        log.info("Found {} reports".format(len(data_by_sample)))
+
         # Find and load any fastp reports
         self.fastp_data = dict()
         self.fastp_duplication_plotdata = dict()
@@ -41,20 +56,10 @@ class MultiqcModule(BaseMultiqcModule):
             self.fastp_qual_plotdata[k] = dict()
             self.fastp_gc_content_data[k] = dict()
             self.fastp_n_content_data[k] = dict()
+        for s_name, parsed_json in data_by_sample.items():
+            self.process_parsed_data(parsed_json, s_name)
 
-        for f in self.find_log_files("fastp", filehandles=True):
-            self.parse_fastp_log(f)
-
-        # Filter to strip out ignored sample names
-        self.fastp_data = self.ignore_samples(self.fastp_data)
-
-        if len(self.fastp_data) == 0:
-            raise UserWarning
-
-        log.info("Found {} reports".format(len(self.fastp_data)))
-
-        # Write parsed report data to a file
-        ## Parse whole JSON to save all its content
+        # Save entire original parsed JSON
         self.write_data_file(self.fastp_all_data, "multiqc_fastp")
 
         # General Stats Table
@@ -147,25 +152,59 @@ class MultiqcModule(BaseMultiqcModule):
         except RuntimeError:
             log.debug("No data found for 'N content' plot")
 
-    def parse_fastp_log(self, f):
+    def parse_fastp_log(self, f) -> Tuple[Optional[str], Dict]:
         """Parse the JSON output from fastp and save the summary statistics"""
         try:
             parsed_json = json.load(f["f"])
-            parsed_json["command"]
-        except:
-            log.warning("Could not parse fastp JSON: '{}'".format(f["fn"]))
-            return None
+        except json.JSONDecodeError as e:
+            log.warning(f"Could not parse fastp JSON: '{f['fn']}': {e}, skipping sample")
+            return None, {}
+        if not isinstance(parsed_json, dict) or "command" not in parsed_json:
+            log.warning(f"Could not find 'command' field in JSON: '{f['fn']}', skipping sample")
+            return None, {}
 
-        # Fetch a sample name from the command
-        s_name = f["s_name"]
-        cmd = parsed_json["command"].split()
-        for i, v in enumerate(cmd):
-            if v == "-i":
-                s_name = self.clean_s_name(cmd[i + 1], f)
-        if s_name == "fastp":
-            log.warning("Could not parse sample name from fastp command: {}".format(f["fn"]))
+        s_name = None
+        if getattr(config, "fastp", {}).get("s_name_filenames", False):
+            s_name = f["s_name"]
+
+        if s_name is None:
+            # Parse the "command" line usually found in the JSON, and use the first input
+            # FastQ file name to fetch the sample name.
+            cmd = parsed_json["command"].strip()
+            # On caveat is that the command won't have file names escaped properly,
+            # so we need some special logic to account for names with spaces:
+            # "fastp -c -g -y -i Sample 1 1.fastq.gz -o ..."
+            # "fastp -c -g -y --in1 Sample 1 1.fastq.gz --out1 ..."
+            # "fastp -c -g -y --in1 Sample 1 1.fastq.gz --in2 Sample 1_2.fastq.gz --out1 ..."
+            #
+            # Using a regex that extracts everything between "-i " or "--in1 " and " -".
+            # It still won't work exactly right for file names with dashes following a
+            # space, but that's a pretty rare case, and will still extract something
+            # meaningful.
+            s_names = []
+            m = re.search(r"(-i|--in1)\s(.+?)(?:\s-|$)", cmd)
+            if m:
+                s_names.append(m.group(2))
+                # Second input for paired end?
+                m = re.search(r"--in2\s(.+?)(?:\s-|$)", cmd)
+                if m:
+                    s_names.append(m.group(1))
+                s_name = self.clean_s_name(s_names, f)
+            else:
+                s_name = f["s_name"]
+                log.warning(
+                    f"Could not parse sample name from the fastp command:\n{cmd}\n"
+                    f"Falling back to extracting it from the file name: "
+                    f"\"{f['fn']}\" -> \"{s_name}\""
+                )
+                return None, {}
 
         self.add_data_source(f, s_name)
+        return s_name, parsed_json
+
+    def process_parsed_data(self, parsed_json: Dict, s_name: str):
+        """Process the JSON extracted from logs"""
+
         self.fastp_data[s_name] = {}
         self.fastp_duplication_plotdata[s_name] = {}
         self.fastp_insert_size_data[s_name] = {}
@@ -180,13 +219,13 @@ class MultiqcModule(BaseMultiqcModule):
             for k in parsed_json["filtering_result"]:
                 self.fastp_data[s_name]["filtering_result_{}".format(k)] = float(parsed_json["filtering_result"][k])
         except KeyError:
-            log.debug("fastp JSON did not have 'filtering_result' key: '{}'".format(f["fn"]))
+            log.debug(f"fastp JSON did not have 'filtering_result' key: '{s_name}'")
 
         # Parse duplication
         try:
             self.fastp_data[s_name]["pct_duplication"] = float(parsed_json["duplication"]["rate"] * 100.0)
         except KeyError:
-            log.debug("fastp JSON did not have a 'duplication' key: '{}'".format(f["fn"]))
+            log.debug(f"fastp JSON did not have a 'duplication' key: '{s_name}'")
 
         # Parse after_filtering
         try:
@@ -195,7 +234,7 @@ class MultiqcModule(BaseMultiqcModule):
                     parsed_json["summary"]["after_filtering"][k]
                 )
         except KeyError:
-            log.debug("fastp JSON did not have a 'summary'-'after_filtering' keys: '{}'".format(f["fn"]))
+            log.debug(f"fastp JSON did not have a 'summary'-'after_filtering' keys: '{s_name}'")
 
         # Parse data required to calculate Pct reads surviving
         try:
@@ -203,7 +242,7 @@ class MultiqcModule(BaseMultiqcModule):
                 parsed_json["summary"]["before_filtering"]["total_reads"]
             )
         except KeyError:
-            log.debug("Could not find pre-filtering # reads: '{}'".format(f["fn"]))
+            log.debug(f"Could not find pre-filtering # reads: '{s_name}'")
 
         try:
             self.fastp_data[s_name]["pct_surviving"] = (
@@ -211,7 +250,7 @@ class MultiqcModule(BaseMultiqcModule):
                 / self.fastp_data[s_name]["before_filtering_total_reads"]
             ) * 100.0
         except (KeyError, ZeroDivisionError) as e:
-            log.debug("Could not calculate 'pct_surviving' ({}): {}".format(e.__class__.__name__, f["fn"]))
+            log.debug(f"Could not calculate 'pct_surviving' ({e.__class__.__name__}): {s_name}")
 
         # Parse adapter_cutting
         try:
@@ -221,7 +260,7 @@ class MultiqcModule(BaseMultiqcModule):
                 except (ValueError, TypeError):
                     pass
         except KeyError:
-            log.debug("fastp JSON did not have a 'adapter_cutting' key, skipping: '{}'".format(f["fn"]))
+            log.debug(f"fastp JSON did not have a 'adapter_cutting' key, skipping: '{s_name}'")
 
         try:
             self.fastp_data[s_name]["pct_adapter"] = (
@@ -229,7 +268,7 @@ class MultiqcModule(BaseMultiqcModule):
                 / self.fastp_data[s_name]["before_filtering_total_reads"]
             ) * 100.0
         except (KeyError, ZeroDivisionError) as e:
-            log.debug("Could not calculate 'pct_adapter' ({}): {}".format(e.__class__.__name__, f["fn"]))
+            log.debug(f"Could not calculate 'pct_adapter' ({e.__class__.__name__}): {s_name}")
 
         # Duplication rate plot data
         try:
@@ -243,7 +282,7 @@ class MultiqcModule(BaseMultiqcModule):
             for i, v in enumerate(parsed_json["duplication"]["histogram"]):
                 self.fastp_duplication_plotdata[s_name][i + 1] = (float(v) / float(total_reads)) * 100.0
         except KeyError:
-            log.debug("No duplication rate plot data: {}".format(f["fn"]))
+            log.debug(f"No duplication rate plot data: {s_name}")
 
         # Insert size plot data
         try:
@@ -261,7 +300,7 @@ class MultiqcModule(BaseMultiqcModule):
                 if i <= max_i:
                     self.fastp_insert_size_data[s_name][i + 1] = (float(v) / float(total_reads)) * 100.0
         except KeyError:
-            log.debug("No insert size plot data: {}".format(f["fn"]))
+            log.debug(f"No insert size plot data: {s_name}")
 
         for k in ["read1_before_filtering", "read2_before_filtering", "read1_after_filtering", "read2_after_filtering"]:
             # Read quality data
@@ -269,7 +308,7 @@ class MultiqcModule(BaseMultiqcModule):
                 for i, v in enumerate(parsed_json[k]["quality_curves"]["mean"]):
                     self.fastp_qual_plotdata[k][s_name][i + 1] = float(v)
             except KeyError:
-                log.debug("Read quality {} not found: {}".format(k, f["fn"]))
+                log.debug(f"Read quality {k} not found: {s_name}")
 
             # GC and N content plots
             try:
@@ -278,7 +317,7 @@ class MultiqcModule(BaseMultiqcModule):
                 for i, v in enumerate(parsed_json[k]["content_curves"]["N"]):
                     self.fastp_n_content_data[k][s_name][i + 1] = float(v) * 100.0
             except KeyError:
-                log.debug("Content curve data {} not found: {}".format(k, f["fn"]))
+                log.debug(f"Content curve data {k} not found: {s_name}")
 
         # Remove empty dicts
         if len(self.fastp_data[s_name]) == 0:
@@ -291,70 +330,76 @@ class MultiqcModule(BaseMultiqcModule):
             del self.fastp_all_data[s_name]
         # Don't delete dicts with subkeys, messes up multi-panel plots
 
+        # Add software version if available
+        # Note: this was added to fastp JSON output in v0.22, so it won't be available in older versions
+        if "fastp_version" in parsed_json["summary"]:
+            self.add_software_version(parsed_json["summary"]["fastp_version"], s_name)
+
     def fastp_general_stats_table(self):
         """Take the parsed stats from the fastp report and add it to the
         General Statistics table at the top of the report"""
 
-        headers = OrderedDict()
-        headers["pct_duplication"] = {
-            "title": "% Duplication",
-            "description": "Duplication rate before filtering",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "RdYlGn-rev",
-        }
-        headers["after_filtering_q30_rate"] = {
-            "title": "% > Q30",
-            "description": "Percentage of reads > Q30 after filtering",
-            "min": 0,
-            "max": 100,
-            "modify": lambda x: x * 100.0,
-            "scale": "GnBu",
-            "suffix": "%",
-            "hidden": True,
-        }
-        headers["after_filtering_q30_bases"] = {
-            "title": "{} Q30 bases".format(config.base_count_prefix),
-            "description": "Bases > Q30 after filtering ({})".format(config.base_count_desc),
-            "min": 0,
-            "modify": lambda x: x * config.base_count_multiplier,
-            "scale": "GnBu",
-            "shared_key": "base_count",
-            "hidden": True,
-        }
-        headers["filtering_result_passed_filter_reads"] = {
-            "title": "{} Reads After Filtering".format(config.read_count_prefix),
-            "description": "Total reads after filtering ({})".format(config.read_count_desc),
-            "min": 0,
-            "scale": "Blues",
-            "modify": lambda x: x * config.read_count_multiplier,
-            "shared_key": "read_count",
-        }
-        headers["after_filtering_gc_content"] = {
-            "title": "GC content",
-            "description": "GC content after filtering",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "Blues",
-            "modify": lambda x: x * 100.0,
-        }
-        headers["pct_surviving"] = {
-            "title": "% PF",
-            "description": "Percent reads passing filter",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "BuGn",
-        }
-        headers["pct_adapter"] = {
-            "title": "% Adapter",
-            "description": "Percentage adapter-trimmed reads",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "RdYlGn-rev",
+        headers = {
+            "pct_duplication": {
+                "title": "% Duplication",
+                "description": "Duplication rate before filtering",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "RdYlGn-rev",
+            },
+            "after_filtering_q30_rate": {
+                "title": "% > Q30",
+                "description": "Percentage of reads > Q30 after filtering",
+                "min": 0,
+                "max": 100,
+                "modify": lambda x: x * 100.0,
+                "scale": "GnBu",
+                "suffix": "%",
+                "hidden": True,
+            },
+            "after_filtering_q30_bases": {
+                "title": "{} Q30 bases".format(config.base_count_prefix),
+                "description": "Bases > Q30 after filtering ({})".format(config.base_count_desc),
+                "min": 0,
+                "modify": lambda x: x * config.base_count_multiplier,
+                "scale": "GnBu",
+                "shared_key": "base_count",
+                "hidden": True,
+            },
+            "filtering_result_passed_filter_reads": {
+                "title": "{} Reads After Filtering".format(config.read_count_prefix),
+                "description": "Total reads after filtering ({})".format(config.read_count_desc),
+                "min": 0,
+                "scale": "Blues",
+                "modify": lambda x: x * config.read_count_multiplier,
+                "shared_key": "read_count",
+            },
+            "after_filtering_gc_content": {
+                "title": "GC content",
+                "description": "GC content after filtering",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "Blues",
+                "modify": lambda x: x * 100.0,
+            },
+            "pct_surviving": {
+                "title": "% PF",
+                "description": "Percent reads passing filter",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "BuGn",
+            },
+            "pct_adapter": {
+                "title": "% Adapter",
+                "description": "Percentage adapter-trimmed reads",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "RdYlGn-rev",
+            },
         }
 
         self.general_stats_addcols(self.fastp_data, headers)
@@ -362,13 +407,14 @@ class MultiqcModule(BaseMultiqcModule):
     def fastp_filtered_reads_chart(self):
         """Function to generate the fastp filtered reads bar plot"""
         # Specify the order of the different possible categories
-        keys = OrderedDict()
-        keys["filtering_result_passed_filter_reads"] = {"name": "Passed Filter"}
-        keys["filtering_result_low_quality_reads"] = {"name": "Low Quality"}
-        keys["filtering_result_too_many_N_reads"] = {"name": "Too Many N"}
-        keys["filtering_result_low_complexity_reads"] = {"name": "Low Complexity"}
-        keys["filtering_result_too_short_reads"] = {"name": "Too Short"}
-        keys["filtering_result_too_long_reads"] = {"name": "Too Long"}
+        keys = {
+            "filtering_result_passed_filter_reads": {"name": "Passed Filter"},
+            "filtering_result_low_quality_reads": {"name": "Low Quality"},
+            "filtering_result_too_many_N_reads": {"name": "Too Many N"},
+            "filtering_result_low_complexity_reads": {"name": "Low Complexity"},
+            "filtering_result_too_short_reads": {"name": "Too Short"},
+            "filtering_result_too_long_reads": {"name": "Too Long"},
+        }
 
         # Config for the plot
         pconfig = {
