@@ -16,20 +16,21 @@ import math
 import os
 import re
 import zipfile
-from collections import OrderedDict
+from collections import Counter
 
 from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
-from multiqc.plots import bargraph, heatmap, linegraph
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
+from multiqc.plots import bargraph, heatmap, linegraph, table
 from multiqc.utils import report
 
 # Initialise the logger
 log = logging.getLogger(__name__)
 
+VERSION_REGEX = r"FastQC\t([\d\.]+)"
+
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="FastQC",
@@ -65,8 +66,18 @@ class MultiqcModule(BaseMultiqcModule):
             # FastQC zip files should have just one directory inside, containing report
             d_name = fqc_zip.namelist()[0]
             try:
-                with fqc_zip.open(os.path.join(d_name, "fastqc_data.txt")) as fh:
-                    r_data = fh.read().decode("utf8")
+                path = os.path.join(d_name, "fastqc_data.txt")
+                with fqc_zip.open(path) as fh:
+                    r_data = fh.read()
+                    try:
+                        r_data = r_data.decode("utf8")
+                    except UnicodeDecodeError as e:
+                        log.debug(f"Could not parse {path} as Unicode: {e}, attempting the latin-1 encoding")
+                        try:
+                            r_data = r_data.decode("latin-1")
+                        except Exception as e:
+                            log.warning(f"Error reading FastQC data file {path}: {e}. Skipping sample {s_name}.")
+                            continue
                     self.parse_fastqc_report(r_data, s_name, f)
             except KeyError:
                 log.warning("Error - can't find fastqc_raw_data.txt in {}".format(f))
@@ -74,7 +85,7 @@ class MultiqcModule(BaseMultiqcModule):
         # Filter to strip out ignored sample names
         self.fastqc_data = self.ignore_samples(self.fastqc_data)
         if len(self.fastqc_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.fastqc_data)))
 
@@ -147,17 +158,21 @@ class MultiqcModule(BaseMultiqcModule):
         section = None
         s_headers = None
         self.dup_keys = []
-        for l in file_contents.splitlines():
-            if l == ">>END_MODULE":
+        for line in file_contents.splitlines():
+            if line.startswith("##FastQC"):
+                version_match = re.search(VERSION_REGEX, line)
+                if version_match:
+                    self.add_software_version(version_match.group(1), s_name)
+            if line == ">>END_MODULE":
                 section = None
                 s_headers = None
-            elif l.startswith(">>"):
-                (section, status) = l[2:].split("\t", 1)
+            elif line.startswith(">>"):
+                (section, status) = line[2:].split("\t", 1)
                 section = section.lower().replace(" ", "_")
                 self.fastqc_data[s_name]["statuses"][section] = status
             elif section is not None:
-                if l.startswith("#"):
-                    s_headers = l[1:].split("\t")
+                if line.startswith("#"):
+                    s_headers = line[1:].split("\t")
                     # Special case: Total Deduplicated Percentage header line
                     if s_headers[0] == "Total Deduplicated Percentage":
                         self.fastqc_data[s_name]["basic_statistics"].append(
@@ -171,9 +186,9 @@ class MultiqcModule(BaseMultiqcModule):
                         self.fastqc_data[s_name][section] = list()
 
                 elif s_headers is not None:
-                    s = l.split("\t")
+                    s = line.split("\t")
                     row = dict()
-                    for (i, v) in enumerate(s):
+                    for i, v in enumerate(s):
                         v.replace("NaN", "0")
                         try:
                             v = float(v)
@@ -265,59 +280,60 @@ class MultiqcModule(BaseMultiqcModule):
             # Zero reads
             hide_seq_length = True
 
-        headers = OrderedDict()
-        headers["percent_duplicates"] = {
-            "title": "% Dups",
-            "description": "% Duplicate Reads",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "RdYlGn-rev",
-        }
-        headers["percent_gc"] = {
-            "title": "% GC",
-            "description": "Average % GC Content",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "Set1",
-            "format": "{:,.0f}",
-        }
-        headers["avg_sequence_length"] = {
-            "title": "Average Read Length",
-            "description": "Average Read Length (bp)",
-            "min": 0,
-            "suffix": " bp",
-            "scale": "RdYlGn",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["median_sequence_length"] = {
-            "title": "Median Read Length",
-            "description": "Median Read Length (bp)",
-            "min": 0,
-            "suffix": " bp",
-            "scale": "RdYlGn",
-            "format": "{:,.0f}",
-            "hidden": hide_seq_length,
-        }
-        headers["percent_fails"] = {
-            "title": "% Failed",
-            "description": "Percentage of modules failed in FastQC report (includes those not plotted here)",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "Reds",
-            "format": "{:,.0f}",
-            "hidden": True,
-        }
-        headers["total_sequences"] = {
-            "title": "{} Seqs".format(config.read_count_prefix),
-            "description": "Total Sequences ({})".format(config.read_count_desc),
-            "min": 0,
-            "scale": "Blues",
-            "modify": lambda x: x * config.read_count_multiplier,
-            "shared_key": "read_count",
+        headers = {
+            "percent_duplicates": {
+                "title": "% Dups",
+                "description": "% Duplicate Reads",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "RdYlGn-rev",
+            },
+            "percent_gc": {
+                "title": "% GC",
+                "description": "Average % GC Content",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "PuRd",
+                "format": "{:,.0f}",
+            },
+            "avg_sequence_length": {
+                "title": "Average Read Length",
+                "description": "Average Read Length (bp)",
+                "min": 0,
+                "suffix": " bp",
+                "scale": "RdYlGn",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "median_sequence_length": {
+                "title": "Median Read Length",
+                "description": "Median Read Length (bp)",
+                "min": 0,
+                "suffix": " bp",
+                "scale": "RdYlGn",
+                "format": "{:,.0f}",
+                "hidden": hide_seq_length,
+            },
+            "percent_fails": {
+                "title": "% Failed",
+                "description": "Percentage of modules failed in FastQC report (includes those not plotted here)",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "Reds",
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "total_sequences": {
+                "title": "{} Seqs".format(config.read_count_prefix),
+                "description": "Total Sequences ({})".format(config.read_count_desc),
+                "min": 0,
+                "scale": "Blues",
+                "modify": lambda x: x * config.read_count_multiplier,
+                "shared_key": "read_count",
+            },
         }
         self.general_stats_addcols(data, headers)
 
@@ -480,7 +496,7 @@ class MultiqcModule(BaseMultiqcModule):
         """Create the epic HTML for the FastQC sequence content heatmap"""
 
         # Prep the data
-        data = OrderedDict()
+        data = {}
         for s_name in sorted(self.fastqc_data.keys()):
             try:
                 data[s_name] = {
@@ -636,11 +652,11 @@ class MultiqcModule(BaseMultiqcModule):
                     theoretical_gc_raw = None
         if theoretical_gc_raw is not None:
             theoretical_gc = list()
-            for l in theoretical_gc_raw.splitlines():
-                if "# FastQC theoretical GC content curve:" in l:
-                    theoretical_gc_name = l[39:]
-                elif not l.startswith("#"):
-                    s = l.split()
+            for line in theoretical_gc_raw.splitlines():
+                if "# FastQC theoretical GC content curve:" in line:
+                    theoretical_gc_name = line[39:]
+                elif not line.startswith("#"):
+                    s = line.split()
                     try:
                         theoretical_gc.append([float(s[0]), float(s[1])])
                     except (TypeError, IndexError):
@@ -767,7 +783,7 @@ class MultiqcModule(BaseMultiqcModule):
             return None
 
         if not multiple_lenths:
-            lengths = "bp , ".join([str(l) for l in list(avg_seq_lengths)])
+            lengths = "bp , ".join([str(line) for line in list(avg_seq_lengths)])
             desc = "All samples have sequences of a single length ({}bp).".format(lengths)
             if len(avg_seq_lengths) > 1:
                 desc += ' See the <a href="#general_stats">General Statistics Table</a>.'
@@ -807,7 +823,7 @@ class MultiqcModule(BaseMultiqcModule):
                 for d in self.fastqc_data[s_name]["sequence_duplication_levels"]:
                     thisdata[d["duplication_level"]] = d["percentage_of_total"]
                     max_dupval = max(max_dupval, d["percentage_of_total"])
-                data[s_name] = OrderedDict()
+                data[s_name] = {}
                 for k in self.dup_keys:
                     try:
                         data[s_name][k] = thisdata[k]
@@ -869,6 +885,9 @@ class MultiqcModule(BaseMultiqcModule):
         """Sum the percentages of overrepresented sequences and display them in a bar plot"""
 
         data = dict()
+        # Count the number of samples where a sequence is overrepresented
+        overrep_by_sample = Counter()
+        overrep_total_cnt = Counter()
         for s_name in self.fastqc_data:
             data[s_name] = dict()
             try:
@@ -879,11 +898,15 @@ class MultiqcModule(BaseMultiqcModule):
                 data[s_name]["total_overrepresented"] = total_pcnt
                 data[s_name]["top_overrepresented"] = max_pcnt
                 data[s_name]["remaining_overrepresented"] = total_pcnt - max_pcnt
+                for d in self.fastqc_data[s_name]["overrepresented_sequences"]:
+                    overrep_by_sample[d["sequence"]] += 1
+                    overrep_total_cnt[d["sequence"]] += int(d["count"])
             except KeyError:
                 if self.fastqc_data[s_name]["statuses"].get("overrepresented_sequences") == "pass":
                     data[s_name]["total_overrepresented"] = 0
                     data[s_name]["top_overrepresented"] = 0
                     data[s_name]["remaining_overrepresented"] = 0
+                    data[s_name]["overrepresented_sequences"] = []
                 else:
                     del data[s_name]
                     log.debug("Couldn't find data for {}, invalid Key".format(s_name))
@@ -892,14 +915,15 @@ class MultiqcModule(BaseMultiqcModule):
             log.debug("overrepresented_sequences not found in FastQC reports")
             return None
 
-        cats = OrderedDict()
-        cats["top_overrepresented"] = {"name": "Top over-represented sequence"}
-        cats["remaining_overrepresented"] = {"name": "Sum of remaining over-represented sequences"}
+        cats = {
+            "top_overrepresented": {"name": "Top overrepresented sequence"},
+            "remaining_overrepresented": {"name": "Sum of remaining overrepresented sequences"},
+        }
 
         # Config for the plot
         pconfig = {
             "id": "fastqc_overrepresented_sequences_plot",
-            "title": "FastQC: Overrepresented sequences",
+            "title": "FastQC: Overrepresented sequences sample summary",
             "ymin": 0,
             "yCeiling": 100,
             "yMinRange": 20,
@@ -920,13 +944,13 @@ class MultiqcModule(BaseMultiqcModule):
             plot_html = bargraph.plot(data, cats, pconfig)
 
         self.add_section(
-            name="Overrepresented sequences",
+            name="Overrepresented sequences by sample",
             anchor="fastqc_overrepresented_sequences",
             description="The total amount of overrepresented sequences found in each library.",
             helptext="""
             FastQC calculates and lists overrepresented sequences in FastQ files. It would not be
             possible to show this for all samples in a MultiQC report, so instead this plot shows
-            the _number of sequences_ categorized as over represented.
+            the _number of sequences_ categorized as overrepresented.
 
             Sometimes, a single sequence  may account for a large number of reads in a dataset.
             To show this, the bars are split into two: the first shows the overrepresented reads
@@ -940,7 +964,7 @@ class MultiqcModule(BaseMultiqcModule):
             sequence is very overrepresented in the set either means that it is highly biologically
             significant, or indicates that the library is contaminated, or not as diverse as you expected._
 
-            _FastQC lists all of the sequences which make up more than 0.1% of the total.
+            _FastQC lists all the sequences which make up more than 0.1% of the total.
             To conserve memory only sequences which appear in the first 100,000 sequences are tracked
             to the end of the file. It is therefore possible that a sequence which is overrepresented
             but doesn't appear at the start of the file for some reason could be missed by this module._
@@ -948,23 +972,91 @@ class MultiqcModule(BaseMultiqcModule):
             plot=plot_html,
         )
 
+        # Add a table of the top overrepresented sequences
+        # Recalculate counts to percentages for readability:
+        total_read_count = sum([int(d["basic_statistics"]["Total Sequences"]) for d in self.fastqc_data.values()])
+        overrep_total_pct = {seq: (cnt / total_read_count) * 100 for seq, cnt in overrep_total_cnt.items()}
+
+        # Top overrepresented sequences across all samples
+        top_n = getattr(config, "fastqc_config", {}).get("top_overrepresented_sequences", 20)
+        by = getattr(config, "fastqc_config", {}).get("top_overrepresented_sequences_by", "samples")
+        if by == "samples":
+            top_seqs = overrep_by_sample.most_common(top_n)
+        else:
+            top_seqs = overrep_total_cnt.most_common(top_n)
+        headers = {
+            "samples": {
+                "title": "Samples",
+                "description": "Number of samples where this sequence is overrepresented",
+                "scale": "Greens",
+                "min": 0,
+                "format": "{:,d}",
+            },
+            "total_count": {
+                "title": "Occurrences",
+                "description": "Total number of occurrences of the sequence (among the samples where the sequence is overrepresented)",
+                "scale": "Blues",
+                "min": 0,
+                "format": "{:,d}",
+            },
+            "total_percent": {
+                "title": "% of all reads",
+                "description": "Total number of occurrences as the percentage of all reads (among samples where the sequence is overrepresented)",
+                "scale": "Blues",
+                "min": 0,
+                "max": 100,
+                "suffix": "%",
+                "format": "{:,.4f}",
+            },
+        }
+        data = {
+            seq: {
+                "sequence": seq,
+                "total_percent": overrep_total_pct[seq],
+                "total_count": overrep_total_cnt[seq],
+                "samples": overrep_by_sample[seq],
+            }
+            for seq, _ in top_seqs
+        }
+
+        ranked_by = (
+            "the number of samples they occur in" if by == "samples" else "the number of occurrences across all samples"
+        )
+        self.add_section(
+            name="Top overrepresented sequences",
+            anchor="fastqc_top_overrepresented_sequences",
+            description=f"""
+            Top overrepresented sequences across all samples. The table shows {top_n} 
+            most overrepresented sequences across all samples, ranked by {ranked_by}.
+            """,
+            plot=table.plot(
+                data,
+                headers,
+                {
+                    "namespace": self.name,
+                    "id": "fastqc_top_overrepresented_sequences_table",
+                    "table_title": "FastQC: Top overrepresented sequences",
+                    "col1_header": "Overrepresented sequence",
+                    "sortRows": False,
+                },
+            ),
+        )
+
     def adapter_content_plot(self):
         """Create the HTML for the FastQC adapter plot"""
 
         data = dict()
         for s_name in self.fastqc_data:
             try:
-                for d in self.fastqc_data[s_name]["adapter_content"]:
-                    pos = self.avg_bp_from_range(d["position"])
-                    for r in self.fastqc_data[s_name]["adapter_content"]:
-                        pos = self.avg_bp_from_range(r["position"])
-                        for a in r.keys():
-                            k = "{} - {}".format(s_name, a)
-                            if a != "position":
-                                try:
-                                    data[k][pos] = r[a]
-                                except KeyError:
-                                    data[k] = {pos: r[a]}
+                for adapters in self.fastqc_data[s_name]["adapter_content"]:
+                    pos = self.avg_bp_from_range(adapters["position"])
+                    for adapter_name, percent in adapters.items():
+                        k = "{} - {}".format(s_name, adapter_name)
+                        if adapter_name != "position":
+                            try:
+                                data[k][pos] = percent
+                            except KeyError:
+                                data[k] = {pos: percent}
             except KeyError:
                 pass
         if len(data) == 0:
@@ -1030,7 +1122,7 @@ class MultiqcModule(BaseMultiqcModule):
         status_numbers = {"pass": 1, "warn": 0.5, "fail": 0.25}
         data = []
         s_names = []
-        status_cats = OrderedDict()
+        status_cats = {}
         for s_name in sorted(self.fastqc_data.keys()):
             s_names.append(s_name)
             for status_cat, status in self.fastqc_data[s_name]["statuses"].items():
