@@ -1,10 +1,7 @@
-#!/usr/bin/env python
-
 """ MultiQC module to parse output from HOMER findpeaks """
 
 import logging
 import os
-from collections import OrderedDict
 
 # Initialise the logger
 log = logging.getLogger(__name__)
@@ -20,31 +17,41 @@ class FindPeaksReportMixin:
 
         # Filter to strip out ignored sample names
         self.homer_findpeaks = self.ignore_samples(self.homer_findpeaks)
+        if not self.homer_findpeaks:
+            return 0
 
-        if len(self.homer_findpeaks) > 0:
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
 
-            # Write parsed report data to a file
-            self.write_data_file(self.homer_findpeaks, "multiqc_homer_findpeaks")
+        # Write parsed report data to a file
+        self.write_data_file(self.homer_findpeaks, "multiqc_homer_findpeaks")
 
-            # General Stats Table
-            stats_headers = OrderedDict()
-            stats_headers["approximate_ip_efficiency"] = {
+        # General Stats Table
+        stats_headers = {
+            "approximate_ip_efficiency": {
                 "title": "% Efficiency",
                 "description": "Approximate IP efficiency",
                 "min": 0,
                 "max": 100,
                 "suffix": "%",
                 "scale": "RdYlGn",
-            }
-            stats_headers["total_peaks"] = {"title": "Total Peaks", "min": 0, "format": "{:,.0f}", "scale": "GnBu"}
-            stats_headers["expected_tags_per_peak"] = {
+            },
+            "total_peaks": {
+                "title": "Total Peaks",
+                "min": 0,
+                "format": "{:,.0f}",
+                "scale": "GnBu",
+            },
+            "expected_tags_per_peak": {
                 "title": "Tags/Peak",
                 "description": "Expected tags per peak",
                 "min": 0,
                 "format": "{:,.0f}",
                 "scale": "PuRd",
-            }
-            self.general_stats_addcols(self.homer_findpeaks, stats_headers, "HOMER findpeaks")
+            },
+        }
+        self.general_stats_addcols(self.homer_findpeaks, stats_headers, "findpeaks")
 
         return len(self.homer_findpeaks)
 
@@ -52,12 +59,12 @@ class FindPeaksReportMixin:
         """Parse HOMER findPeaks file headers."""
         parsed_data = dict()
         s_name = f["s_name"]
-        for l in f["f"]:
+        for line in f["f"]:
             # Start of data
-            if l.strip() and not l.strip().startswith("#"):
+            if line.strip() and not line.strip().startswith("#"):
                 break
             # Automatically parse header lines by = symbol
-            s = l[2:].split("=")
+            s = line[2:].split("=")
             if len(s) > 1:
                 k = s[0].strip().replace(" ", "_").lower()
                 v = s[1].strip().replace("%", "")
