@@ -1,16 +1,14 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """ MultiQC module to parse output files from iVar """
 
-from __future__ import print_function
-from collections import OrderedDict
+
 import logging
 import re
 
 from multiqc import config
-from multiqc.plots import bargraph, heatmap
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
+from multiqc.plots import heatmap
 
 # Initialise the logger
 log = logging.getLogger(__name__)
@@ -18,7 +16,6 @@ log = logging.getLogger(__name__)
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="iVar",
@@ -41,11 +38,15 @@ class MultiqcModule(BaseMultiqcModule):
         # Stop if no files are found
         num_samples = max(len(self.ivar_data), len(self.ivar_primers))
         if num_samples == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         # Log number of reports
         log.info("Found {} reports".format(num_samples))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         # Write parsed data to a file
         self.write_data_file(self.ivar_data, "multiqc_ivar_summary")
         self.write_data_file(self.ivar_primers, "multiqc_ivar_primers")
@@ -71,16 +72,16 @@ class MultiqcModule(BaseMultiqcModule):
         }
         primer_regex = re.compile(r"^(.*)(?:\t+)(\d+$)")
         parsed_data = dict()
-        primers = OrderedDict()
-        for l in f["f"]:
+        primers = dict()
+        for line in f["f"]:
             # Search count regexes for stats
             for k, count_regex in count_regexes.items():
-                count_match = count_regex.search(l)
+                count_match = count_regex.search(line)
                 if count_match:
                     parsed_data[k] = int(count_match.group(1))
 
             # Try to match the primer regex
-            primer_match = primer_regex.search(l)
+            primer_match = primer_regex.search(line)
             if primer_match:
                 primers[primer_match.group(1)] = int(primer_match.group(2))
 
@@ -99,44 +100,45 @@ class MultiqcModule(BaseMultiqcModule):
         """Take the parsed stats from the iVAR report and add it to the
         basic stats table"""
 
-        headers = OrderedDict()
-        headers["reads_too_short_after_trimming"] = {
-            "title": "{} Too short".format(config.read_count_prefix),
-            "description": "Number of reads too short (<30bp) after primer trimming ({})".format(
-                config.read_count_desc
-            ),
-            "scale": "OrRd",
-            "shared_key": "read_counts",
-            "modify": lambda x: x * config.read_count_multiplier,
-        }
-        headers["reads_outside_primer_region"] = {
-            "title": "{} Outside primer".format(config.read_count_prefix),
-            "description": "Number of reads outside the primer region ({})".format(config.read_count_desc),
-            "scale": "YlOrBr",
-            "shared_key": "read_counts",
-            "modify": lambda x: x * config.read_count_multiplier,
-        }
-        headers["trimmed_reads"] = {
-            "title": "{} Primer trimmed".format(config.read_count_prefix),
-            "description": "Total number of reads where primer trimming was performed. ({})".format(
-                config.read_count_desc
-            ),
-            "scale": "Purples",
-            "shared_key": "read_counts",
-            "modify": lambda x: x * config.read_count_multiplier,
-        }
-        headers["mapped_reads"] = {
-            "title": "{} Mapped".format(config.read_count_prefix),
-            "description": "Total number of mapped reads in iVar input. ({})".format(config.read_count_desc),
-            "scale": "PuBu",
-            "shared_key": "read_counts",
-            "modify": lambda x: x * config.read_count_multiplier,
+        headers = {
+            "reads_too_short_after_trimming": {
+                "title": "{} Too short".format(config.read_count_prefix),
+                "description": "Number of reads too short (<30bp) after primer trimming ({})".format(
+                    config.read_count_desc
+                ),
+                "scale": "OrRd",
+                "shared_key": "read_counts",
+                "modify": lambda x: x * config.read_count_multiplier,
+            },
+            "reads_outside_primer_region": {
+                "title": "{} Outside primer".format(config.read_count_prefix),
+                "description": "Number of reads outside the primer region ({})".format(config.read_count_desc),
+                "scale": "YlOrBr",
+                "shared_key": "read_counts",
+                "modify": lambda x: x * config.read_count_multiplier,
+            },
+            "trimmed_reads": {
+                "title": "{} Primer trimmed".format(config.read_count_prefix),
+                "description": "Total number of reads where primer trimming was performed. ({})".format(
+                    config.read_count_desc
+                ),
+                "scale": "Purples",
+                "shared_key": "read_counts",
+                "modify": lambda x: x * config.read_count_multiplier,
+            },
+            "mapped_reads": {
+                "title": "{} Mapped".format(config.read_count_prefix),
+                "description": "Total number of mapped reads in iVar input. ({})".format(config.read_count_desc),
+                "scale": "PuBu",
+                "shared_key": "read_counts",
+                "modify": lambda x: x * config.read_count_multiplier,
+            },
         }
         self.general_stats_addcols(self.ivar_data, headers)
 
     def primer_heatmap(self):
         """Heatmap showing information on each primer found for every sample"""
-        # Top level dict contains sample IDs + OrderedDict(primer, counts)
+        # Top level dict contains sample IDs + dict(primer, counts)
 
         final_data = list()
         final_xcats = list()
