@@ -4,10 +4,9 @@
 import logging
 import os
 import re
-from collections import OrderedDict
 
 from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph
 
 # Initialise the logger
@@ -18,7 +17,6 @@ class MultiqcModule(BaseMultiqcModule):
     """Kallisto module"""
 
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="Kallisto",
@@ -37,10 +35,14 @@ class MultiqcModule(BaseMultiqcModule):
         self.kallisto_data = self.ignore_samples(self.kallisto_data)
 
         if len(self.kallisto_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.kallisto_data)))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         # Write parsed report data to a file
         self.write_data_file(self.kallisto_data, "multiqc_kallisto")
 
@@ -52,26 +54,25 @@ class MultiqcModule(BaseMultiqcModule):
 
     def parse_kallisto_log(self, f):
         s_name = total_reads = paligned_reads = fraglength = None
-        for l in f["f"]:
-
+        for line in f["f"]:
             # Get input filename
-            match = re.search(r"\[quant\] will process (pair|file|sample) 1: (\S+)", l)
+            match = re.search(r"\[quant\] will process (pair|file|sample) 1: (\S+)", line)
             if match:
                 s_name = self.clean_s_name(os.path.basename(match.group(2)), f)
 
             if s_name is not None:
                 # Alignment rates
-                aligned = re.search(r"\[quant\] processed ([\d,]+) reads, ([\d,]+) reads pseudoaligned", l)
+                aligned = re.search(r"\[quant\] processed ([\d,]+) reads, ([\d,]+) reads pseudoaligned", line)
                 if aligned:
                     total_reads = float(aligned.group(1).replace(",", ""))
                     paligned_reads = float(aligned.group(2).replace(",", ""))
 
                 # Paired end fragment lengths
-                flength = re.search(r"\[quant\] estimated average fragment length: ([\d\.]+)", l)
+                flength = re.search(r"\[quant\] estimated average fragment length: ([\d\.]+)", line)
                 if flength:
                     fraglength = float(flength.group(1).replace(",", ""))
 
-                if "quantifying the abundances" in l:
+                if "quantifying the abundances" in line:
                     if s_name in self.kallisto_data:
                         log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                     self.add_data_source(f, s_name)
@@ -92,29 +93,30 @@ class MultiqcModule(BaseMultiqcModule):
         """Take the parsed stats from the Kallisto report and add it to the
         basic stats table at the top of the report"""
 
-        headers = OrderedDict()
-        headers["fragment_length"] = {
-            "title": "Frag Length",
-            "description": "Estimated average fragment length",
-            "min": 0,
-            "suffix": "bp",
-            "scale": "RdYlGn",
-        }
-        headers["percent_aligned"] = {
-            "title": "% Aligned",
-            "description": "% processed reads that were pseudoaligned",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "YlGn",
-        }
-        headers["pseudoaligned_reads"] = {
-            "title": "{} Aligned".format(config.read_count_prefix),
-            "description": "Pseudoaligned reads ({})".format(config.read_count_desc),
-            "min": 0,
-            "scale": "PuRd",
-            "modify": lambda x: x * config.read_count_multiplier,
-            "shared_key": "read_count",
+        headers = {
+            "fragment_length": {
+                "title": "Frag Length",
+                "description": "Estimated average fragment length",
+                "min": 0,
+                "suffix": "bp",
+                "scale": "RdYlGn",
+            },
+            "percent_aligned": {
+                "title": "% Aligned",
+                "description": "% processed reads that were pseudoaligned",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "YlGn",
+            },
+            "pseudoaligned_reads": {
+                "title": "{} Aligned".format(config.read_count_prefix),
+                "description": "Pseudoaligned reads ({})".format(config.read_count_desc),
+                "min": 0,
+                "scale": "PuRd",
+                "modify": lambda x: x * config.read_count_multiplier,
+                "shared_key": "read_count",
+            },
         }
         self.general_stats_addcols(self.kallisto_data, headers)
 
@@ -122,9 +124,10 @@ class MultiqcModule(BaseMultiqcModule):
         """Make the HighCharts HTML to plot the alignment rates"""
 
         # Specify the order of the different possible categories
-        keys = OrderedDict()
-        keys["pseudoaligned_reads"] = {"color": "#437bb1", "name": "Pseudoaligned"}
-        keys["not_pseudoaligned_reads"] = {"color": "#b1084c", "name": "Not aligned"}
+        keys = {
+            "pseudoaligned_reads": {"color": "#437bb1", "name": "Pseudoaligned"},
+            "not_pseudoaligned_reads": {"color": "#b1084c", "name": "Not aligned"},
+        }
 
         # Config for the plot
         config = {
