@@ -1,16 +1,12 @@
-#!/usr/bin/env python
-
 """ MultiQC module to parse output from kraken """
 
-from __future__ import print_function
-from collections import OrderedDict
-import os
+
 import logging
 import re
 
 from multiqc import config
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, heatmap
-from multiqc.modules.base_module import BaseMultiqcModule
 
 # Initialise the logger
 log = logging.getLogger(__name__)
@@ -19,38 +15,44 @@ log = logging.getLogger(__name__)
 class MultiqcModule(BaseMultiqcModule):
     """Kraken module"""
 
-    def __init__(self):
-
-        # Initialise the parent object
+    def __init__(
+        self,
+        name="Kraken",
+        anchor="kraken",
+        href="https://ccb.jhu.edu/software/kraken/",
+        info="is a taxonomic classification tool that uses exact k-mer matches to find the lowest common ancestor (LCA) of a given sequence.",
+        doi="10.1186/gb-2014-15-3-r46",
+        sp_key="kraken",
+    ):
         super(MultiqcModule, self).__init__(
-            name="Kraken",
-            anchor="kraken",
-            href="https://ccb.jhu.edu/software/kraken/",
-            info="is a taxonomic classification tool that uses exact k-mer matches to find the lowest common ancestor (LCA) of a given sequence.",
-            doi="10.1186/gb-2014-15-3-r46",
+            name=name,
+            anchor=anchor,
+            href=href,
+            info=info,
+            doi=doi,
         )
+        self.t_ranks = {
+            "S": "Species",
+            "G": "Genus",
+            "F": "Family",
+            "O": "Order",
+            "C": "Class",
+            "P": "Phylum",
+            "K": "Kingdom",
+            "D": "Domain",
+            "R": "Root",
+            "U": "Unclassified",
+        }
 
-        self.t_ranks = OrderedDict()
-        self.t_ranks["S"] = "Species"
-        self.t_ranks["G"] = "Genus"
-        self.t_ranks["F"] = "Family"
-        self.t_ranks["O"] = "Order"
-        self.t_ranks["C"] = "Class"
-        self.t_ranks["P"] = "Phylum"
-        self.t_ranks["K"] = "Kingdom"
-        self.t_ranks["D"] = "Domain"
-        self.t_ranks["R"] = "Root"
-        # self.t_ranks['U'] = 'Unclassified'
-
-        self.top_n = 5
+        self.top_n = getattr(config, "kraken", {}).get("top_n", 5)
 
         # Find and load any kraken reports
         self.kraken_raw_data = dict()
         new_report_present = False
-        for f in self.find_log_files("kraken", filehandles=True):
-            log_version = self.get_log_version(f)
+        for f in self.find_log_files(sp_key, filehandles=True):
+            log_is_new = self.log_is_new(f)
             f["f"].seek(0)
-            if log_version == "old":
+            if not log_is_new:
                 self.parse_logs(f)
             else:
                 new_report_present = True
@@ -60,14 +62,17 @@ class MultiqcModule(BaseMultiqcModule):
         self.kraken_raw_data = self.ignore_samples(self.kraken_raw_data)
 
         if len(self.kraken_raw_data) == 0:
-            raise UserWarning
-
-        # Data is in wrong format for writing to file
-        # self.write_data_file(self.kraken_raw_data, "kraken")
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.kraken_raw_data)))
 
-        # Sum counts across all samples, so that we can pick top 5
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
+        self.write_data_file(self.kraken_raw_data, f"multiqc_{self.anchor}")
+
+        # Sum counts across all samples, so that we can pick top species
         self.kraken_total_pct = dict()
         self.kraken_total_counts = dict()
         self.kraken_sample_total_readcounts = dict()
@@ -75,22 +80,24 @@ class MultiqcModule(BaseMultiqcModule):
         self.sum_sample_counts()
 
         self.general_stats_cols()
-        self.top_five_barplot()
+        self.top_taxa_barplot()
         if new_report_present:
-            self.top_five_duplication_heatmap()
+            self.top_taxa_duplication_heatmap()
 
-    def get_log_version(self, f):
+    @staticmethod
+    def log_is_new(f):
         """Check which version of Kraken report file is used
 
         If 6 fields are used, it's the 'old' log (without distinct minimizer)
         if 8 fields, the new log experimental log (with distinct minimizer)
         """
 
-        for l in f["f"]:
-            if len(l.split()) > 6:
-                return "new"
+        for line in f["f"]:
+            if len(line.split()) > 6:
+                return True
             else:
-                return "old"
+                return False
+        return False
 
     def parse_logs(self, f):
         """
@@ -122,8 +129,8 @@ class MultiqcModule(BaseMultiqcModule):
         # Search regexes for stats
         k2_regex = re.compile(r"^\s{0,2}(\d{1,3}\.\d{1,2})\t(\d+)\t(\d+)\t([\dUDKRPCOFGS-]{1,3})\t(\d+)(\s+)(.+)")
         data = []
-        for l in f["f"]:
-            match = k2_regex.search(l)
+        for line in f["f"]:
+            match = k2_regex.search(line)
             if match:
                 row = {
                     "percent": float(match.group(1)),
@@ -183,11 +190,11 @@ class MultiqcModule(BaseMultiqcModule):
 
         # Search regexes for stats
         k2_regex = re.compile(
-            r"^\s{1,2}(\d{1,2}\.\d{1,2})\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t([UDKPCOFGS-]\d{0,2})\t(\d+)(\s+)(.+)"
+            r"^\s{0,2}(\d{1,3}\.\d{1,2})\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t([URDKPCOFGS-]\d{0,2})\t(\d+)(\s+)(.+)"
         )
         data = []
-        for l in f["f"]:
-            match = k2_regex.search(l)
+        for line in f["f"]:
+            match = k2_regex.search(line)
             if match:
                 row = {
                     "percent": float(match.group(1)),
@@ -202,6 +209,8 @@ class MultiqcModule(BaseMultiqcModule):
                     "classif": match.group(9),
                 }
                 data.append(row)
+            else:
+                log.debug(f"{f['s_name']}: Could not parse line: {line}")
 
         self.kraken_raw_data[f["s_name"]] = data
 
@@ -218,18 +227,17 @@ class MultiqcModule(BaseMultiqcModule):
         # Check that we had some counts for some samples, exit if not
         if total_all_samples == 0:
             log.warning("No samples had any reads")
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
     def sum_sample_counts(self):
         """Sum counts across all samples for kraken data"""
 
         # Sum the percentages for each taxa across all samples
-        # Allows us to pick top-5 for each rank
+        # Allows us to pick the top taxa for each rank
         # Use percentages instead of counts so that deeply-sequences samples
         # are not unfairly over-represented
         for s_name, data in self.kraken_raw_data.items():
             for row in data:
-
                 # Convenience vars that are easier to read
                 rank_code = row["rank_code"]
                 classif = row["classif"]
@@ -257,42 +265,47 @@ class MultiqcModule(BaseMultiqcModule):
         """Add a couple of columns to the General Statistics table"""
 
         # Get top taxa in most specific taxa rank that we have
-        top_five = []
+        top_taxa = []
         top_rank_code = None
         top_rank_name = None
         for rank_code, rank_name in self.t_ranks.items():
             try:
                 sorted_pct = sorted(self.kraken_total_pct[rank_code].items(), key=lambda x: x[1], reverse=True)
                 for classif, pct_sum in sorted_pct[: self.top_n]:
-                    top_five.append(classif)
+                    top_taxa.append(classif)
                 top_rank_code = rank_code
                 top_rank_name = rank_name
                 break
             except KeyError:
                 # No species-level data found etc
                 pass
-        top_one_hkey = "% {}".format(top_five[0])
 
         # Column headers
-        headers = OrderedDict()
-        headers[top_one_hkey] = {
-            "title": top_one_hkey,
-            "description": "Percentage of reads that were the top {} over all samples - {}".format(
-                top_rank_name, top_five[0]
-            ),
-            "suffix": "%",
-            "max": 100,
-            "scale": "PuBuGn",
-        }
-        headers["% Top 5"] = {
-            "title": "% Top 5 {}".format(top_rank_name),
-            "description": "Percentage of reads that were classified by one of the top 5 {} ({})".format(
-                top_rank_name, ", ".join(top_five)
-            ),
-            "suffix": "%",
-            "max": 100,
-            "scale": "PuBu",
-        }
+        headers = dict()
+
+        top_one_hkey = None
+
+        # don't include top-N % in general stats if all is unclassified.
+        # unclassified is included separately, so also don't include twice
+        if top_rank_code != "U":
+            top_one_hkey = "% {}".format(top_taxa[0])
+            headers[top_one_hkey] = {
+                "title": top_one_hkey,
+                "description": "Percentage of reads that were the top {} over all samples - {}".format(
+                    top_rank_name, top_taxa[0]
+                ),
+                "suffix": "%",
+                "max": 100,
+                "scale": "PuBuGn",
+            }
+            headers["% Top"] = {
+                "title": f"% Top {self.top_n} {top_rank_name}",
+                "description": f"Percentage of reads that were classified by one of the top-{self.top_n} {top_rank_name} ({', '.join(top_taxa)})",
+                "suffix": "%",
+                "max": 100,
+                "scale": "PuBu",
+            }
+
         headers["% Unclassified"] = {
             "title": "% Unclassified",
             "description": "Percentage of reads that were unclassified",
@@ -312,33 +325,29 @@ class MultiqcModule(BaseMultiqcModule):
                     percent = 0
                 if row["rank_code"] == "U":
                     tdata[s_name]["% Unclassified"] = percent
-                if row["rank_code"] == top_rank_code and row["classif"] in top_five:
-                    tdata[s_name]["% Top 5"] = percent + tdata[s_name].get("% Top 5", 0)
-                if row["rank_code"] == top_rank_code and row["classif"] == top_five[0]:
+                if row["rank_code"] == top_rank_code and row["classif"] in top_taxa:
+                    tdata[s_name]["% Top"] = percent + tdata[s_name].get("% Top", 0)
+                if row["rank_code"] == top_rank_code and row["classif"] == top_taxa[0]:
                     tdata[s_name][top_one_hkey] = percent
 
-            if top_one_hkey not in tdata[s_name]:
+            if top_one_hkey is not None and top_one_hkey not in tdata[s_name]:
                 tdata[s_name][top_one_hkey] = 0
 
         self.general_stats_addcols(tdata, headers)
 
-    def top_five_barplot(self):
-        """Add a bar plot showing the top-5 from each taxa rank"""
+    def top_taxa_barplot(self):
+        """Add a bar plot showing the top-N from each taxa rank"""
 
         pd = []
         cats = list()
-        pconfig = {
-            "id": "kraken-topfive-plot",
-            "title": "Kraken 2: Top taxa",
-            "ylab": "Number of fragments",
-            "data_labels": list(self.t_ranks.values()),
-        }
+        # Keeping track of encountered codes to display only tabs with available data
+        found_rank_codes = set()
 
-        for rank_code, rank_name in self.t_ranks.items():
-            rank_cats = OrderedDict()
+        for rank_code in self.t_ranks:
+            rank_cats = dict()
             rank_data = dict()
 
-            # Loop through the summed tax percentages to get the top 5 across all samples
+            # Loop through the summed tax percentages to get the top-N across all samples
             try:
                 sorted_pct = sorted(self.kraken_total_pct[rank_code].items(), key=lambda x: x[1], reverse=True)
             except KeyError:
@@ -357,13 +366,17 @@ class MultiqcModule(BaseMultiqcModule):
                         rank_data[s_name] = dict()
                     if s_name not in counts_shown:
                         counts_shown[s_name] = 0
+
                     for row in d:
                         if row["rank_code"] == rank_code:
-                            if row["classif"] == classif:
-                                if classif not in rank_data[s_name]:
-                                    rank_data[s_name][classif] = 0
-                                rank_data[s_name][classif] += row["counts_rooted"]
-                                counts_shown[s_name] += row["counts_rooted"]
+                            found_rank_codes.add(rank_code)
+                            # unclassified are handled separately
+                            if row["rank_code"] != "U":
+                                if row["classif"] == classif:
+                                    if classif not in rank_data[s_name]:
+                                        rank_data[s_name][classif] = 0
+                                    rank_data[s_name][classif] += row["counts_rooted"]
+                                    counts_shown[s_name] += row["counts_rooted"]
 
             # Add in unclassified reads and "other" - we presume from other species etc.
             for s_name, d in self.kraken_raw_data.items():
@@ -388,13 +401,20 @@ class MultiqcModule(BaseMultiqcModule):
             cats.append(rank_cats)
             pd.append(rank_data)
 
+        pconfig = {
+            "id": f"{self.anchor}-top-n-plot",
+            "title": f"{self.name}: Top taxa",
+            "ylab": "Number of fragments",
+            "data_labels": [v for k, v in self.t_ranks.items() if k in found_rank_codes],
+        }
+
         self.add_section(
             name="Top taxa",
-            anchor="kraken-topfive",
+            anchor=f"{self.anchor}-top-n",
             description=f"The number of reads falling into the top {self.top_n} taxa across different ranks.",
             helptext=f"""
                 To make this plot, the percentage of each sample assigned to a given taxa is summed across all samples.
-                The counts for these top five taxa are then plotted for each of the 9 different taxa ranks.
+                The counts for these top {self.top_n} taxa are then plotted for each of the 9 different taxa ranks.
                 The unclassified count is always shown across all taxa ranks.
 
                 The total number of reads is approximated by dividing the number of `unclassified` reads by the percentage of
@@ -409,15 +429,20 @@ class MultiqcModule(BaseMultiqcModule):
             plot=bargraph.plot(pd, cats, pconfig),
         )
 
-    def top_five_duplication_heatmap(self):
-        """Add a heatmap showing the minimizer duplication top-5 species"""
+    def top_taxa_duplication_heatmap(self):
+        """Add a heatmap showing the minimizer duplication of the top taxa"""
 
         duplication = list()
-        pconfig = {"id": "kraken-topfive-duplication_plot", "title": f"Kraken 2: Top {self.top_n} species duplication"}
+        pconfig = {
+            "id": f"{self.anchor}-top-duplication_plot",
+            "title": f"{self.name}: Top {self.top_n} species duplication",
+            "square": False,
+            "xcats_samples": False,
+        }
 
         rank_code = "S"
         rank_data = dict()
-        # Loop through the summed tax percentages to get the top 5 across all samples
+        # Loop through the summed tax percentages to get the top taxa across all samples
         try:
             sorted_pct = sorted(self.kraken_total_pct[rank_code].items(), key=lambda x: x[1], reverse=True)
         except KeyError:
@@ -438,18 +463,24 @@ class MultiqcModule(BaseMultiqcModule):
                     rank_data[s_name] = dict()
                 if s_name not in counts_shown:
                     counts_shown[s_name] = 0
-                for row in d:
-                    if row["rank_code"] == rank_code:
-                        if row["classif"] == classif:
-                            if classif not in rank_data[s_name]:
-                                rank_data[s_name][classif] = 0
-                            try:
-                                rank_data[s_name][classif] = row["minimizer_duplication"]
-                            except KeyError:
-                                del rank_data[s_name]
-                                if not showed_warning:
-                                    log.warning("Kraken2 reports of different versions were found")
-                                    showed_warning = True
+
+                if classif not in rank_data[s_name]:
+                    rank_data[s_name][classif] = None
+
+                try:
+                    row = next(row for row in d if row["rank_code"] == rank_code and row["classif"] == classif)
+                except StopIteration:
+                    # if nothing is found at the rank + classification, leave as 0
+                    continue
+
+                try:
+                    rank_data[s_name][classif] = row["minimizer_duplication"]
+                except KeyError:
+                    del rank_data[s_name]
+                    if not showed_warning:
+                        log.warning("Kraken2 reports of different versions were found")
+                        showed_warning = True
+
         # Strip empty samples
         for sample, vals in dict(rank_data).items():
             if len(vals) == 0:
@@ -463,7 +494,7 @@ class MultiqcModule(BaseMultiqcModule):
 
         self.add_section(
             name="Duplication rate of top species",
-            anchor="kraken-duplication-topfive",
+            anchor=f"{self.anchor}-duplication-topfive",
             description=f"The duplication rate of minimizer falling into the top {self.top_n} species",
             helptext=f"""
                 To make this plot, the minimizer duplication rate is computed for the top {self.top_n} most abundant species in all samples.
