@@ -2,9 +2,8 @@
 
 
 import logging
-from collections import OrderedDict
 
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph
 
 # Initialise the logger
@@ -17,7 +16,6 @@ class MultiqcModule(BaseMultiqcModule):
     """
 
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="OptiType",
@@ -47,69 +45,73 @@ class MultiqcModule(BaseMultiqcModule):
         self.optitype_data = self.ignore_samples(self.optitype_data)
 
         if len(self.optitype_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.optitype_data)))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         # Write parsed report data to a file
         self.write_data_file(self.optitype_data, "multiqc_optitype")
 
         # Create OptiType overview table
-        self.addSummaryMetrics()
+        self.add_summary_metrics()
 
         # Create bar graph of subtype sample counts
         self.summary_barplot()
 
-    def addSummaryMetrics(self):
+    def add_summary_metrics(self):
         """Take the parsed entries from OptiType and add them to the main plot"""
         # A1, A2, B1, B2, C1, C2, Reads, Objective
-        headers = OrderedDict()
-
-        headers["A1"] = {
-            "title": "HLA-A1",
-            "description": "First HLA-A allele",
-            "scale": False,
-        }
-        headers["A2"] = {
-            "title": "HLA-A2",
-            "description": "Second HLA-A allele",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["B1"] = {
-            "title": "HLA-B1",
-            "description": "First HLA-B allele",
-            "scale": False,
-        }
-        headers["B2"] = {
-            "title": "HLA-B2",
-            "description": "Second HLA-B allele",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["C1"] = {
-            "title": "HLA-C1",
-            "description": "First HLA-C allele",
-            "scale": False,
-        }
-        headers["C2"] = {
-            "title": "HLA-C2",
-            "description": "Second HLA-C allele",
-            "scale": False,
-            "hidden": True,
-        }
-        headers["Reads"] = {
-            "title": "Reads",
-            "description": "Number of reads covering the HLA",
-            "scale": "YlGnBu",
-            "hidden": True,
-            "format": "{:,.0f}",
-        }
-        headers["Objective"] = {
-            "title": "Objective Score",
-            "description": "Score of the objective function for the prediction.",
-            "scale": "BuGn",
-            "hidden": True,
+        headers = {
+            "A1": {
+                "title": "HLA-A1",
+                "description": "First HLA-A allele",
+                "scale": False,
+            },
+            "A2": {
+                "title": "HLA-A2",
+                "description": "Second HLA-A allele",
+                "scale": False,
+                "hidden": True,
+            },
+            "B1": {
+                "title": "HLA-B1",
+                "description": "First HLA-B allele",
+                "scale": False,
+            },
+            "B2": {
+                "title": "HLA-B2",
+                "description": "Second HLA-B allele",
+                "scale": False,
+                "hidden": True,
+            },
+            "C1": {
+                "title": "HLA-C1",
+                "description": "First HLA-C allele",
+                "scale": False,
+            },
+            "C2": {
+                "title": "HLA-C2",
+                "description": "Second HLA-C allele",
+                "scale": False,
+                "hidden": True,
+            },
+            "Reads": {
+                "title": "Reads",
+                "description": "Number of reads covering the HLA",
+                "scale": "YlGnBu",
+                "hidden": True,
+                "format": "{:,.0f}",
+            },
+            "Objective": {
+                "title": "Objective Score",
+                "description": "Score of the objective function for the prediction.",
+                "scale": "BuGn",
+                "hidden": True,
+            },
         }
         self.general_stats_addcols(self.optitype_data, headers)
 
