@@ -1,13 +1,11 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """ MultiQC module to parse output from phantompeakqualtools """
 
-from __future__ import print_function
-from collections import OrderedDict
+
 import logging
 
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 
 # Initialise the logger
 log = logging.getLogger(__name__)
@@ -15,7 +13,6 @@ log = logging.getLogger(__name__)
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="phantompeakqualtools",
@@ -35,11 +32,15 @@ class MultiqcModule(BaseMultiqcModule):
 
         # Warning when no files are found
         if len(self.phantompeakqualtools_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         # Log
         log.info("Found {} logs".format(len(self.phantompeakqualtools_data)))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None, f["s_name"])
+
         # Write parsed data to a file
         self.write_data_file(self.phantompeakqualtools_data, "multiqc_phantompeakqualtools")
 
@@ -50,8 +51,8 @@ class MultiqcModule(BaseMultiqcModule):
     def parse_phantompeakqualtools(self, f):
         parsed_data = {}
         lines = f["f"].splitlines()
-        for l in lines:
-            s = l.split("\t")
+        for line in lines:
+            s = line.split("\t")
             parsed_data["Estimated_Fragment_Length_bp"] = int(s[2].split(",")[0])
             parsed_data["NSC"] = float(s[8])
             parsed_data["RSC"] = float(s[9])
@@ -64,27 +65,28 @@ class MultiqcModule(BaseMultiqcModule):
     # Report fragment length, NSC and RSC in general stat table
     def phantompeakqualtools_general_stats(self):
         """Add columns to General Statistics table"""
-        headers = OrderedDict()
-        headers["Estimated_Fragment_Length_bp"] = {
-            "title": "Frag Length",
-            "description": "Estimated fragment length (bp)",
-            "min": 0,
-            "format": "{:,.0f}",
-        }
-        headers["NSC"] = {
-            "title": "NSC",
-            "description": "Normalized strand cross-correlation",
-            "max": 10,
-            "min": 0,
-            "format": "{:,.2f}",
-            "scale": "RdYlGn-rev",
-        }
-        headers["RSC"] = {
-            "title": "RSC",
-            "description": "Relative strand cross-correlation",
-            "max": 10,
-            "min": 0,
-            "format": "{:,.2f}",
-            "scale": "RdYlBu-rev",
+        headers = {
+            "Estimated_Fragment_Length_bp": {
+                "title": "Frag Length",
+                "description": "Estimated fragment length (bp)",
+                "min": 0,
+                "format": "{:,.0f}",
+            },
+            "NSC": {
+                "title": "NSC",
+                "description": "Normalized strand cross-correlation",
+                "max": 10,
+                "min": 0,
+                "format": "{:,.2f}",
+                "scale": "RdYlGn-rev",
+            },
+            "RSC": {
+                "title": "RSC",
+                "description": "Relative strand cross-correlation",
+                "max": 10,
+                "min": 0,
+                "format": "{:,.2f}",
+                "scale": "RdYlBu-rev",
+            },
         }
         self.general_stats_addcols(self.phantompeakqualtools_data, headers)
