@@ -4,7 +4,6 @@
 import logging
 import math
 import re
-from collections import OrderedDict
 
 from multiqc import config
 from multiqc.plots import linegraph
@@ -20,6 +19,7 @@ def parse_reports(self):
     self.qualimap_bamqc_genome_results = dict()
     for f in self.find_log_files("qualimap/bamqc/genome_results"):
         parse_genome_results(self, f)
+
     self.qualimap_bamqc_genome_results = self.ignore_samples(self.qualimap_bamqc_genome_results)
     if len(self.qualimap_bamqc_genome_results) > 0:
         self.write_data_file(self.qualimap_bamqc_genome_results, "multiqc_qualimap_bamqc_genome_results")
@@ -54,9 +54,13 @@ def parse_reports(self):
     if num_parsed == 0:
         return 0
 
+    # Superfluous function call to confirm that it is used in this module
+    # Replace None with actual version if it is available
+    self.add_software_version(None)
+
     try:
         covs = config.qualimap_config["general_stats_coverage"]
-        assert type(covs) == list
+        assert isinstance(covs, list)
         assert len(covs) > 0
         covs = [str(i) for i in covs]
         log.debug("Custom Qualimap thresholds: {}".format(", ".join([i for i in covs])))
@@ -115,7 +119,7 @@ def parse_genome_results(self, f):
             for k, r in regexes.get(section, {}).items():
                 r_search = re.search(r, line)
                 if r_search:
-                    if "\d" in r:
+                    if r"\d" in r:
                         try:
                             d[k] = float(r_search.group(1).replace(",", ""))
                         except ValueError:
@@ -135,12 +139,17 @@ def parse_genome_results(self, f):
     try:
         self.general_stats_data[s_name]["total_reads"] = d["total_reads"]
         self.general_stats_data[s_name]["mapped_reads"] = d["mapped_reads"]
-        d["percentage_aligned"] = (d["mapped_reads"] / d["total_reads"]) * 100
-        self.general_stats_data[s_name]["percentage_aligned"] = d["percentage_aligned"]
-        self.general_stats_data[s_name]["general_error_rate"] = d["general_error_rate"] * 100
+        self.general_stats_data[s_name]["general_error_rate"] = d["general_error_rate"] * 100.0
         self.general_stats_data[s_name]["mean_coverage"] = d["mean_coverage"]
         self.general_stats_data[s_name]["regions_size"] = d["regions_size"]
         self.general_stats_data[s_name]["regions_mapped_reads"] = d["regions_mapped_reads"]
+        try:
+            d["percentage_aligned"] = (d["mapped_reads"] / d["total_reads"]) * 100.0
+            self.general_stats_data[s_name]["percentage_aligned"] = d["percentage_aligned"]
+            d["percentage_aligned_on_target"] = (d["regions_mapped_reads"] / d["mapped_reads"]) * 100.0
+            self.general_stats_data[s_name]["percentage_aligned_on_target"] = d["percentage_aligned_on_target"]
+        except ZeroDivisionError:
+            pass
     except KeyError:
         pass
 
@@ -153,15 +162,15 @@ def parse_genome_results(self, f):
 
 def parse_coverage(self, f):
     """Parse the contents of the Qualimap BamQC Coverage Histogram file"""
-    # Get the sample name from the parent parent directory
+    # Get the sample name from the parent directory
     # Typical path: <sample name>/raw_data_qualimapReport/coverage_histogram.txt
     s_name = self.get_s_name(f)
 
     d = dict()
-    for l in f["f"]:
-        if l.startswith("#"):
+    for line in f["f"]:
+        if line.startswith("#"):
             continue
-        coverage, count = l.split(None, 1)
+        coverage, count = line.split(None, 1)
         coverage = int(round(float(coverage)))
         count = float(count)
         d[coverage] = count
@@ -191,21 +200,18 @@ def parse_coverage(self, f):
 
 def parse_insert_size(self, f):
     """Parse the contents of the Qualimap BamQC Insert Size Histogram file"""
-    # Get the sample name from the parent parent directory
+    # Get the sample name from the parent directory
     # Typical path: <sample name>/raw_data_qualimapReport/insert_size_histogram.txt
     s_name = self.get_s_name(f)
 
     d = dict()
-    zero_insertsize = 0
-    for l in f["f"]:
-        if l.startswith("#"):
+    for line in f["f"]:
+        if line.startswith("#"):
             continue
-        insertsize, count = l.split(None, 1)
+        insertsize, count = line.split(None, 1)
         insertsize = int(round(float(insertsize)))
         count = float(count) / 1000000
-        if insertsize == 0:
-            zero_insertsize = count
-        else:
+        if insertsize != 0:
             d[insertsize] = count
 
     # Find median without importing anything to do it for us
@@ -229,7 +235,7 @@ def parse_insert_size(self, f):
 
 def parse_gc_dist(self, f):
     """Parse the contents of the Qualimap BamQC Mapped Reads GC content distribution file"""
-    # Get the sample name from the parent parent directory
+    # Get the sample name from the parent directory
     # Typical path: <sample name>/raw_data_qualimapReport/mapped_reads_gc-content_distribution.txt
     s_name = self.get_s_name(f)
 
@@ -237,13 +243,13 @@ def parse_gc_dist(self, f):
     reference_species = None
     reference_d = dict()
     avg_gc = 0
-    for l in f["f"]:
-        if l.startswith("#"):
-            sections = l.strip("\n").split("\t", 3)
+    for line in f["f"]:
+        if line.startswith("#"):
+            sections = line.strip("\n").split("\t", 3)
             if len(sections) > 2:
                 reference_species = sections[2]
             continue
-        sections = l.strip("\n").split("\t", 3)
+        sections = line.strip("\n").split("\t", 3)
         gc = int(round(float(sections[0])))
         content = float(sections[1])
         avg_gc += gc * content
@@ -536,7 +542,7 @@ def report_sections(self):
 def general_stats_headers(self):
     try:
         hidecovs = config.qualimap_config["general_stats_coverage_hidden"]
-        assert type(hidecovs) == list
+        assert isinstance(hidecovs, list)
         log.debug("Hiding Qualimap thresholds: {}".format(", ".join([i for i in hidecovs])))
     except (AttributeError, TypeError, KeyError, AssertionError):
         hidecovs = [1, 5, 10, 50]
@@ -548,7 +554,7 @@ def general_stats_headers(self):
         "max": 100,
         "min": 0,
         "suffix": "%",
-        "scale": "Set1",
+        "scale": "PuRd",
         "format": "{:,.0f}",
     }
     self.general_stats_headers["median_insert_size"] = {
@@ -582,6 +588,38 @@ def general_stats_headers(self):
         "suffix": "X",
         "scale": "BuPu",
     }
+    self.general_stats_headers["percentage_aligned_on_target"] = {
+        "title": "% On target",
+        "description": "% mapped reads on target region",
+        "max": 100,
+        "min": 0,
+        "suffix": "%",
+        "scale": "YlGn",
+    }
+    self.general_stats_headers["regions_size"] = {
+        "title": "{} Region size".format(config.read_count_prefix),
+        "description": "Size of target region",
+        "suffix": " bp",
+        "scale": "PuBuGn",
+        "hidden": True,
+    }
+    self.general_stats_headers["regions_mapped_reads"] = {
+        "title": "{} On target".format(config.read_count_prefix),
+        "description": "Number of mapped reads on target region ({})".format(config.read_count_desc),
+        "scale": "RdYlGn",
+        "shared_key": "read_count",
+        "hidden": True,
+    }
+    self.general_stats_headers["general_error_rate"] = {
+        "title": "Error rate",
+        "description": "Alignment error rate. Total edit distance (SAM NM field) over the number of mapped bases",
+        "max": 100,
+        "min": 0,
+        "suffix": "%",
+        "scale": "OrRd",
+        "format": "{0:.2f}",
+        "hidden": True,
+    }
     self.general_stats_headers["percentage_aligned"] = {
         "title": "% Aligned",
         "description": "% mapped reads",
@@ -604,35 +642,11 @@ def general_stats_headers(self):
         "shared_key": "read_count",
         "hidden": True,
     }
-    self.general_stats_headers["general_error_rate"] = {
-        "title": "Error rate",
-        "description": "Alignment error rate. Total edit distance (SAM NM field) over the number of mapped bases",
-        "max": 100,
-        "min": 0,
-        "suffix": "%",
-        "scale": "OrRd",
-        "format": "{0:.2f}",
-        "hidden": True,
-    }
-    self.general_stats_headers["regions_size"] = {
-        "title": "{} Region size".format(config.read_count_prefix),
-        "description": "Size of target region",
-        "suffix": " bp",
-        "scale": "PuBuGn",
-        "hidden": True,
-    }
-    self.general_stats_headers["regions_mapped_reads"] = {
-        "title": "{} Aligned".format(config.read_count_prefix),
-        "description": "Number of mapped reads on target region ({})".format(config.read_count_desc),
-        "scale": "RdYlGn",
-        "shared_key": "read_count",
-        "hidden": True,
-    }
 
 
 def _calculate_bases_within_thresholds(bases_by_depth, total_size, depth_thresholds):
-    bases_within_threshs = OrderedDict((depth, 0) for depth in depth_thresholds)
-    rates_within_threshs = OrderedDict((depth, None) for depth in depth_thresholds)
+    bases_within_threshs = {depth: 0 for depth in depth_thresholds}
+    rates_within_threshs = {depth: None for depth in depth_thresholds}
 
     dt = sorted(depth_thresholds, reverse=True)
     c = 0
