@@ -2,11 +2,10 @@
 
 
 import logging
-from collections import OrderedDict
 
-from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, linegraph
+from multiqc.utils import config
 
 # Initialise the logger
 log = logging.getLogger(__name__)
@@ -18,7 +17,6 @@ class MultiqcModule(BaseMultiqcModule):
     """
 
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="Rsem",
@@ -40,10 +38,14 @@ class MultiqcModule(BaseMultiqcModule):
         self.rsem_mapped_data = self.ignore_samples(self.rsem_mapped_data)
 
         if len(self.rsem_mapped_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.rsem_mapped_data)))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         # Write parsed report data to a file
         self.write_data_file(self.rsem_mapped_data, "multiqc_rsem")
 
@@ -63,8 +65,8 @@ class MultiqcModule(BaseMultiqcModule):
         data = dict()
         multimapping_hist = dict()
         in_hist = False
-        for l in f["f"].splitlines():
-            s = l.split()
+        for line in f["f"].splitlines():
+            s = line.split()
             if len(s) > 3:
                 # Line: N0 N1 N2 N_tot
                 # N0, number of unalignable reads;
@@ -118,14 +120,15 @@ class MultiqcModule(BaseMultiqcModule):
     def rsem_stats_table(self):
         """Take the parsed stats from the rsem report and add them to the
         basic stats table at the top of the report"""
-        headers = OrderedDict()
-        headers["alignable_percent"] = {
-            "title": "% Alignable".format(config.read_count_prefix),
-            "description": "% Alignable reads".format(config.read_count_desc),
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "YlGn",
+        headers = {
+            "alignable_percent": {
+                "title": "% Alignable, {}".format(config.read_count_prefix),
+                "description": "% Alignable reads, {}".format(config.read_count_desc),
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "YlGn",
+            }
         }
         self.general_stats_addcols(self.rsem_mapped_data, headers)
 
@@ -133,11 +136,12 @@ class MultiqcModule(BaseMultiqcModule):
         """Make the rsem assignment rates plot"""
 
         # Plot categories
-        keys = OrderedDict()
-        keys["Unique"] = {"color": "#437bb1", "name": "Aligned uniquely to a gene"}
-        keys["Multi"] = {"color": "#e63491", "name": "Aligned to multiple genes"}
-        keys["Filtered"] = {"color": "#b1084c", "name": "Filtered due to too many alignments"}
-        keys["Unalignable"] = {"color": "#7f0000", "name": "Unalignable reads"}
+        keys = {
+            "Unique": {"color": "#437bb1", "name": "Aligned uniquely to a gene"},
+            "Multi": {"color": "#e63491", "name": "Aligned to multiple genes"},
+            "Filtered": {"color": "#b1084c", "name": "Filtered due to too many alignments"},
+            "Unalignable": {"color": "#7f0000", "name": "Unalignable reads"},
+        }
 
         # Config for the plot
         config = {
