@@ -1,19 +1,18 @@
 import logging
 import re
-from collections import OrderedDict
 
-from multiqc import config
 from multiqc.plots import bargraph
 
 log = logging.getLogger(__name__)
 
+VERSION_REGEX = r"sambamba ([\d\.]+)"
+
 
 class SambambaMarkdupMixin:
 
     """Find and parse Sambamba Markdup output log files and calculate duplication rate"""
 
     def parse_sambamba_markdup(self):
-
         # Clean sample name from 'markdup_sample_1' to 'sample_1'
         # Find and load sambamba logs to markdup_data.
         # Regex for key phrases and calculate duplicate rate.
@@ -23,7 +22,6 @@ class SambambaMarkdupMixin:
         self.markdup_data = dict()
 
         for f in self.find_log_files("sambamba/markdup"):
-
             if f["s_name"] in self.markdup_data:
                 log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f["fn"], f["s_name"]))
 
@@ -35,6 +33,11 @@ class SambambaMarkdupMixin:
             # filter away samples if MultiQC user does not want them
             self.markdup_data = self.ignore_samples(self.markdup_data)
 
+            # add sambamba version to software table
+            version_match = re.search(VERSION_REGEX, f["f"])
+            if version_match:
+                self.add_software_version(version_match.group(1), f["s_name"])
+
             # add results to multiqc_sources.txt
             self.add_data_source(f)
 
@@ -107,7 +110,6 @@ class SambambaMarkdupMixin:
         return d
 
     def markdup_general_stats_table(self):
-
         """
         Take parsed stats from sambamba markdup to general stats table at the top of the report.
 
@@ -128,18 +130,17 @@ class SambambaMarkdupMixin:
             self.general_stats_data[s_name]["duplicate_rate"] = data["duplicate_rate"]
 
     def markdup_section(self):
-
         """
         Add markdup statistics as bar graph to multiQC report.
         """
 
         # plot these categories, but not duplicate rate.
-        cats = OrderedDict()
-        cats["total_sorted_paired_end_reads"] = {"name": "Total Sorted Paired End Reads"}
-        cats["total_single_end_reads"] = {"name": "Total Single End Reads"}
-        cats["total_single_unmatched_reads"] = {"name": "Total Single Unmatched Reads"}
-        cats["duplicate_reads"] = {"name": "Total Duplicate Reads"}
-
+        cats = {
+            "total_sorted_paired_end_reads": {"name": "Total Sorted Paired End Reads"},
+            "total_single_end_reads": {"name": "Total Single End Reads"},
+            "total_single_unmatched_reads": {"name": "Total Single Unmatched Reads"},
+            "duplicate_reads": {"name": "Total Duplicate Reads"},
+        }
         config = {
             "id": "SambambaMarkdupBargraph",
             "title": "Sambamba Markdup: Duplicate Counts",
