@@ -3,7 +3,6 @@
 
 import logging
 import re
-from collections import OrderedDict
 
 from multiqc import config
 from multiqc.plots import beeswarm
@@ -28,29 +27,34 @@ class FlagstatReportMixin:
         # Filter to strip out ignored sample names
         self.samtools_flagstat = self.ignore_samples(self.samtools_flagstat)
 
-        if len(self.samtools_flagstat) > 0:
+        if len(self.samtools_flagstat) == 0:
+            return 0
 
-            # Write parsed report data to a file (restructure first)
-            self.write_data_file(self.samtools_flagstat, "multiqc_samtools_flagstat")
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
 
-            # General Stats Table
-            flagstats_headers = OrderedDict()
-            flagstats_headers["flagstat_total"] = {
+        # Write parsed report data to a file (restructure first)
+        self.write_data_file(self.samtools_flagstat, "multiqc_samtools_flagstat")
+
+        # General Stats Table
+        flagstats_headers = {
+            "flagstat_total": {
                 "title": "{} Reads".format(config.read_count_prefix),
                 "description": "Total reads in the bam file ({})".format(config.read_count_desc),
                 "min": 0,
                 "modify": lambda x: x * config.read_count_multiplier,
                 "shared_key": "read_count",
                 "hidden": True,
-            }
-            flagstats_headers["mapped_passed"] = {
+            },
+            "mapped_passed": {
                 "title": "{} Reads Mapped".format(config.read_count_prefix),
                 "description": "Reads Mapped in the bam file ({})".format(config.read_count_desc),
                 "min": 0,
                 "modify": lambda x: x * config.read_count_multiplier,
                 "shared_key": "read_count",
-            }
-            flagstats_headers["mapped_passed_pct"] = {
+            },
+            "mapped_passed_pct": {
                 "title": "% Reads Mapped",
                 "description": "% Reads Mapped in the bam file",
                 "min": 0,
@@ -58,48 +62,49 @@ class FlagstatReportMixin:
                 "suffix": "%",
                 "scale": "RdYlGn",
                 "hidden": True,
-            }
-            self.general_stats_addcols(self.samtools_flagstat, flagstats_headers)
+            },
+        }
+        self.general_stats_addcols(self.samtools_flagstat, flagstats_headers)
 
-            # Make dot plot of counts
-            keys = OrderedDict()
-            reads = {
-                "min": 0,
-                "modify": lambda x: float(x) * config.read_count_multiplier,
-                "suffix": "{} reads".format(config.read_count_prefix),
-                "decimalPlaces": 2,
-                "shared_key": "read_count",
-            }
-            keys["flagstat_total"] = dict(reads, title="Total Reads")
-            keys["total_passed"] = dict(reads, title="Total Passed QC")
-            keys["mapped_passed"] = dict(reads, title="Mapped")
+        # Make dot plot of counts
+        keys = {}
+        reads = {
+            "min": 0,
+            "modify": lambda x: float(x) * config.read_count_multiplier,
+            "suffix": "{} reads".format(config.read_count_prefix),
+            "decimalPlaces": 2,
+            "shared_key": "read_count",
+        }
+        keys["flagstat_total"] = dict(reads, title="Total Reads")
+        keys["total_passed"] = dict(reads, title="Total Passed QC")
+        keys["mapped_passed"] = dict(reads, title="Mapped")
 
-            if any(v.get("secondary_passed") for v in self.samtools_flagstat.values()):
-                keys["secondary_passed"] = dict(reads, title="Secondary Alignments")
+        if any(v.get("secondary_passed") for v in self.samtools_flagstat.values()):
+            keys["secondary_passed"] = dict(reads, title="Secondary Alignments")
 
-            if any(v.get("supplementary_passed") for v in self.samtools_flagstat.values()):
-                keys["supplementary_passed"] = dict(reads, title="Supplementary Alignments")
+        if any(v.get("supplementary_passed") for v in self.samtools_flagstat.values()):
+            keys["supplementary_passed"] = dict(reads, title="Supplementary Alignments")
 
-            keys["duplicates_passed"] = dict(reads, title="Duplicates")
-            keys["paired in sequencing_passed"] = dict(reads, title="Paired in Sequencing")
-            keys["properly paired_passed"] = dict(reads, title="Properly Paired")
-            keys["with itself and mate mapped_passed"] = dict(
-                reads, title="Self and mate mapped", description="Reads with itself and mate mapped"
-            )
-            keys["singletons_passed"] = dict(reads, title="Singletons")
-            keys["with mate mapped to a different chr_passed"] = dict(
-                reads, title="Mate mapped to diff chr", description="Mate mapped to different chromosome"
-            )
-            keys["with mate mapped to a different chr (mapQ >= 5)_passed"] = dict(
-                reads, title="Diff chr (mapQ >= 5)", description="Mate mapped to different chromosome (mapQ >= 5)"
-            )
+        keys["duplicates_passed"] = dict(reads, title="Duplicates")
+        keys["paired in sequencing_passed"] = dict(reads, title="Paired in Sequencing")
+        keys["properly paired_passed"] = dict(reads, title="Properly Paired")
+        keys["with itself and mate mapped_passed"] = dict(
+            reads, title="Self and mate mapped", description="Reads with itself and mate mapped"
+        )
+        keys["singletons_passed"] = dict(reads, title="Singletons")
+        keys["with mate mapped to a different chr_passed"] = dict(
+            reads, title="Mate mapped to diff chr", description="Mate mapped to different chromosome"
+        )
+        keys["with mate mapped to a different chr (mapQ >= 5)_passed"] = dict(
+            reads, title="Diff chr (mapQ >= 5)", description="Mate mapped to different chromosome (mapQ >= 5)"
+        )
 
-            self.add_section(
-                name="Samtools Flagstat",
-                anchor="samtools-flagstat",
-                description="This module parses the output from <code>samtools flagstat</code>. All numbers in millions.",
-                plot=beeswarm.plot(self.samtools_flagstat, keys, {"id": "samtools-flagstat-dp"}),
-            )
+        self.add_section(
+            name="Samtools Flagstat",
+            anchor="samtools-flagstat",
+            description="This module parses the output from <code>samtools flagstat</code>. All numbers in millions.",
+            plot=beeswarm.plot(self.samtools_flagstat, keys, {"id": "samtools-flagstat-dp"}),
+        )
 
         # Return the number of logs that were found
         return len(self.samtools_flagstat)
