@@ -2,10 +2,9 @@
 
 
 import logging
-from collections import OrderedDict
 
 from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph
 
 # Initialise the logger
@@ -14,7 +13,6 @@ log = logging.getLogger(__name__)
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="sargasso",
@@ -38,10 +36,14 @@ class MultiqcModule(BaseMultiqcModule):
         self.sargasso_data = self.ignore_samples(self.sargasso_data)
 
         if len(self.sargasso_data) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} reports".format(len(self.sargasso_files)))
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         # Write parsed report data to a file
         self.write_data_file(self.sargasso_data, "multiqc_sargasso")
 
@@ -51,16 +53,13 @@ class MultiqcModule(BaseMultiqcModule):
         # Assignment bar plot
         self.add_section(plot=self.sargasso_chart())
 
-        # log.info('done')
-
     def parse_sargasso_logs(self, f):
         """Parse the sargasso log file."""
         species_name = list()
         items = list()
-        header = list()
         is_first_line = True
-        for l in f["f"].splitlines():
-            s = l.split(",")
+        for line in f["f"].splitlines():
+            s = line.split(",")
             # Check that this actually is a Sargasso file
             if is_first_line and s[0] != "Sample":
                 return None
@@ -103,7 +102,6 @@ class MultiqcModule(BaseMultiqcModule):
         self.sargasso_keys = items
 
         for idx, f_name in enumerate(self.sargasso_data.keys()):
-
             # Reorganised parsed data for this sample
             # Collect total READ count number
             self.sargasso_data[f_name]["Total"] = 0
@@ -123,22 +121,23 @@ class MultiqcModule(BaseMultiqcModule):
         """Take the parsed stats from the sargasso report and add them to the
         basic stats table at the top of the report"""
 
-        headers = OrderedDict()
-        headers["sargasso_percent_assigned"] = {
-            "title": "% Assigned",
-            "description": "Sargasso % Assigned reads",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "RdYlGn",
-        }
-        headers["Assigned-Reads"] = {
-            "title": "{} Assigned".format(config.read_count_prefix),
-            "description": "Sargasso Assigned reads ({})".format(config.read_count_desc),
-            "min": 0,
-            "scale": "PuBu",
-            "modify": lambda x: float(x) * config.read_count_multiplier,
-            "shared_key": "read_count",
+        headers = {
+            "sargasso_percent_assigned": {
+                "title": "% Assigned",
+                "description": "Sargasso % Assigned reads",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "RdYlGn",
+            },
+            "Assigned-Reads": {
+                "title": "{} Assigned".format(config.read_count_prefix),
+                "description": "Sargasso Assigned reads ({})".format(config.read_count_desc),
+                "min": 0,
+                "scale": "PuBu",
+                "modify": lambda x: float(x) * config.read_count_multiplier,
+                "shared_key": "read_count",
+            },
         }
         self.general_stats_addcols(self.sargasso_data, headers)
 
