@@ -4,10 +4,9 @@
 import logging
 import os
 import re
-from collections import OrderedDict
 
 from multiqc import config
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph
 
 # Initialise the logger
@@ -16,7 +15,6 @@ log = logging.getLogger(__name__)
 
 class MultiqcModule(BaseMultiqcModule):
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="SortMeRNA",
@@ -36,23 +34,29 @@ class MultiqcModule(BaseMultiqcModule):
         self.sortmerna = self.ignore_samples(self.sortmerna)
 
         if len(self.sortmerna) == 0:
-            raise UserWarning
+            raise ModuleNoSamplesFound
 
         log.info("Found {} logs".format(len(self.sortmerna)))
+
         self.write_data_file(self.sortmerna, "multiqc_sortmerna")
 
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
+
         # Get custom table header, default to 'rRNA'
         tab_header = getattr(config, "sortmerna", {}).get("seqname", "% rRNA")
 
         # Add rRNA rate to the general stats table
-        headers = OrderedDict()
-        headers["rRNA_pct"] = {
-            "title": tab_header,
-            "description": "Percentage of reads matched to a SortMeRNA database",
-            "max": 100,
-            "min": 0,
-            "suffix": "%",
-            "scale": "OrRd",
+        headers = {
+            "rRNA_pct": {
+                "title": tab_header,
+                "description": "Percentage of reads matched to a SortMeRNA database",
+                "max": 100,
+                "min": 0,
+                "suffix": "%",
+                "scale": "OrRd",
+            }
         }
         self.general_stats_addcols(self.sortmerna, headers)
 
@@ -66,26 +70,26 @@ class MultiqcModule(BaseMultiqcModule):
         db_number = 0
         err = False
 
-        for l in f["f"]:
-            if "Reads file" in l:
-                parts = re.split(":|=", l)
+        for line in f["f"]:
+            if "Reads file" in line:
+                parts = re.split(r":|=", line)
                 s_name = self.clean_s_name(parts[-1], f)
                 self.sortmerna[s_name] = dict()
-            if "Results:" in l and not post_results_start:  # old versions
+            if "Results:" in line and not post_results_start:  # old versions
                 post_results_start = True
-            if "Total reads = " in l and not post_results_start:  # v4.2.0 onwards
+            if "Total reads = " in line and not post_results_start:  # v4.2.0 onwards
                 post_results_start = True
             if not post_results_start:
                 continue
             if post_results_start and not post_database_start:
-                if "Total reads =" in l:
-                    m = re.search("\d+", l)
+                if "Total reads =" in line:
+                    m = re.search("\d+", line)
                     if m:
                         self.sortmerna[s_name]["total"] = int(m.group())
                     else:
                         err = True
-                elif "Total reads passing" in l:
-                    m = re.search("\d+", l)
+                elif "Total reads passing" in line:
+                    m = re.search("\d+", line)
                     if m:
                         self.sortmerna[s_name]["rRNA"] = int(m.group())
                         self.sortmerna[s_name]["rRNA_pct"] = (
@@ -93,8 +97,8 @@ class MultiqcModule(BaseMultiqcModule):
                         )
                     else:
                         err = True
-                elif "Total reads failing" in l:
-                    m = re.search("\d+", l)
+                elif "Total reads failing" in line:
+                    m = re.search("\d+", line)
                     if m:
                         self.sortmerna[s_name]["non_rRNA"] = int(m.group())
                         self.sortmerna[s_name]["non_rRNA_pct"] = (
@@ -103,13 +107,12 @@ class MultiqcModule(BaseMultiqcModule):
                     else:
                         err = True
             if post_database_start:
-
                 # Stop when we hit empty lines
-                if not l.strip():
+                if not line.strip():
                     break
 
                 db_number = db_number + 1
-                parts = re.split("\t+", l.strip())
+                parts = re.split("\t+", line.strip())
                 if len(parts) == 2:
                     db = os.path.splitext(os.path.basename(parts[0]))[0]
                     pct = float(parts[1].replace("%", ""))
@@ -119,7 +122,7 @@ class MultiqcModule(BaseMultiqcModule):
                 else:
                     err = True
 
-            if "By database:" in l or "Coverage by database:" in l:
+            if "By database:" in line or "Coverage by database:" in line:
                 post_database_start = True
         if err:
             log.warning("Error parsing data in: " + s_name)
@@ -134,7 +137,7 @@ class MultiqcModule(BaseMultiqcModule):
         metrics = set()
         for sample in self.sortmerna:
             for key in self.sortmerna[sample]:
-                if not key in ["total", "rRNA", "non_rRNA"] and not "_pct" in key:
+                if key not in ["total", "rRNA", "non_rRNA"] and "_pct" not in key:
                     metrics.add(key)
 
         for key in metrics:
