@@ -1,3 +1,3 @@
-from __future__ import absolute_import
-
 from .whatshap import MultiqcModule
+
+__all__ = ["MultiqcModule"]
