@@ -1,22 +1,24 @@
 """ MultiQC module to parse output from WhatsHap """
 
 import logging
-from collections import OrderedDict, defaultdict
+from collections import defaultdict
 
-from multiqc.modules.base_module import BaseMultiqcModule
+from multiqc.modules.base_module import BaseMultiqcModule, ModuleNoSamplesFound
 from multiqc.plots import bargraph, table
 
 # Initialise the logger
 log = logging.getLogger(__name__)
 
 
+ALL_CHROM = "ALL"
+
+
 class MultiqcModule(BaseMultiqcModule):
     """
     WhatsHap module class, parses WhatsHap output.
     """
 
     def __init__(self):
-
         # Initialise the parent object
         super(MultiqcModule, self).__init__(
             name="WhatsHap",
@@ -42,9 +44,12 @@ class MultiqcModule(BaseMultiqcModule):
         # Filter to strip out ignored sample names
         self.whatshap_stats = self.ignore_samples(self.whatshap_stats)
 
-        # Raise UserWarning if we didn't find any data
         if not self.whatshap_stats:
-            raise UserWarning
+            raise ModuleNoSamplesFound
+
+        # Superfluous function call to confirm that it is used in this module
+        # Replace None with actual version if it is available
+        self.add_software_version(None)
 
         # Write parsed report data to a file
         self.write_data_file(self.whatshap_stats, "multiqc_whatshap_stats")
@@ -62,12 +67,12 @@ class MultiqcModule(BaseMultiqcModule):
     def parse_whatshap_stats(self, logfile):
         """Parse WhatsHap stats file"""
 
-        def process_data(data):
+        def parse_numeric_values(data):
             """
             Process the data to convert it to the proper format
 
             In practice, this only involves the following changes:
-                - Convert all numeric values to integers
+                - Convert all numeric values to integers or floats
                 - Replace 'nan' values with zero
             """
             for key, value in data.items():
@@ -75,7 +80,10 @@ class MultiqcModule(BaseMultiqcModule):
                 try:
                     data[key] = int(data[key])
                 except ValueError:
-                    pass
+                    try:
+                        data[key] = float(data[key])
+                    except ValueError:
+                        pass
 
                 # Replace 'nan' with zero
                 if value == "nan":
@@ -86,14 +94,14 @@ class MultiqcModule(BaseMultiqcModule):
         # we are parsing an empty file
         sample = logfile["s_name"]
         # Get the header and remove the "#" character from the first field
-        header = next(file_content).strip().split()
+        header = next(file_content).strip().split("\t")
         header[0] = header[0].lstrip("#")
 
         # The results from this log file, a dictionary for every chromosome
         results = defaultdict(dict)
         # Parse the lines that have the data
         for line in file_content:
-            spline = line.strip().split()
+            spline = line.strip().split("\t")
             data = {field: value for field, value in zip(header, spline)}
 
             # Remove the sample and chromsome from the data
@@ -101,25 +109,63 @@ class MultiqcModule(BaseMultiqcModule):
             chromosome = data.pop("chromosome")
 
             # Process the remaining data fields
-            process_data(data)
-
-            # Calculate the fraction of heterozygous variants that were phased
-            try:
-                frac_het_phased = data["phased"] / data["heterozygous_variants"]
-            except ZeroDivisionError:
-                frac_het_phased = 0
-            data["frac_het_phased"] = frac_het_phased
+            parse_numeric_values(data)
 
             # Insert the current line under chromosome
             results[chromosome] = data
 
         # If we were parsing an empty file, we return dummy data (basically all
         # zero) so the sample statistics are consistent, which eases parsing
-        # later on. WhatsHap uses the fictional 'ALL' chromsome to store the
+        # later on. WhatsHap uses the fictional 'ALL' chromosome to store the
         # summary for all results, so we re-use that to store all zero's
         if not results:
             results = dict()
-            results["ALL"] = defaultdict(int)
+            results[ALL_CHROM] = defaultdict(int)
+
+        # If ALL chromosome is not present, and there are multiple contigs available,
+        # e.g. in case of a truncated input file, we summarize available contigs
+        # as a fictional 'ALL' chromosome.
+        if ALL_CHROM not in results and len(results) > 1:
+            log.warning(
+                f"Could not find the 'ALL' chromosome for {sample}. Make sure that the input is not truncated. MultiQC will summarize information for all available chromosomes instead."
+            )
+            sum_fields = {
+                "variants",
+                "phased",
+                "unphased",
+                "singletons",
+                "blocks",
+                "bp_per_block_sum",
+                "heterozygous_variants",
+                "heterozygous_snvs",
+                "phased_snvs",
+            }
+            avg_fields = {
+                "bp_per_block_avg",
+                "variant_per_block_avg",
+                "bp_per_block_avg",
+            }
+            results[ALL_CHROM] = defaultdict(int)
+            for chrom in results:
+                if chrom == ALL_CHROM:
+                    continue
+                for field in results[chrom]:
+                    val = results[chrom][field]
+                    if val:
+                        if field in sum_fields:
+                            results[ALL_CHROM][field] += int(results[chrom][field])
+                        elif field in avg_fields:
+                            results[ALL_CHROM][field] += float(results[chrom][field]) * results[chrom]["blocks"]
+            for field in avg_fields:
+                results[ALL_CHROM][field] /= results[ALL_CHROM]["blocks"]
+
+        # Calculate the fraction of heterozygous variants that were phased
+        for chrom, data in results.items():
+            try:
+                frac_het_phased = data["phased"] / data["heterozygous_variants"]
+            except ZeroDivisionError:
+                frac_het_phased = 0
+            results[chrom]["frac_het_phased"] = frac_het_phased
 
         # Clean the sample name
         sample = self.clean_s_name(sample, logfile["root"])
@@ -141,7 +187,7 @@ class MultiqcModule(BaseMultiqcModule):
         if len(sample_stats) == 1:
             summary_field = list(sample_stats)[0]
         else:
-            summary_field = "ALL"
+            summary_field = ALL_CHROM
         return summary_field
 
     def whatshap_add_general_stats(self):
@@ -150,52 +196,44 @@ class MultiqcModule(BaseMultiqcModule):
         # Store the configuration for each column, this is also used to
         # determine which columns to add to the general statistics table
         # https://whatshap.readthedocs.io/en/latest/guide.html#the-tsv-statistics-format
-        general_stats_headers = OrderedDict(
-            [
-                (
-                    "frac_het_phased",
-                    {
-                        "id": "perc_het_phased",
-                        "title": "% Phased Variants",
-                        "description": """Fraction of heterozygous variants
+        general_stats_headers = {
+            "frac_het_phased": {
+                "id": "perc_het_phased",
+                "title": "% Phased Variants",
+                "description": """Fraction of heterozygous variants
                                           that could be phased.
                                         """,
-                        "format": "{:,.0f}",
-                        "suffix": "%",
-                        "min": 0,
-                        "max": 100,
-                        "modify": lambda x: x * 100,
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "bp_per_block_avg",
-                    {
-                        "id": "bp_per_block_avg",
-                        "title": "Avg bp per Block",
-                        "description": """Description of the distribution of non-singleton
+                "format": "{:,.0f}",
+                "suffix": "%",
+                "min": 0,
+                "max": 100,
+                "modify": lambda x: x * 100,
+                "scale": "GnBu",
+                "hidden": False,
+            },
+            "bp_per_block_avg": {
+                "id": "bp_per_block_avg",
+                "title": "Avg bp per Block",
+                "description": """Description of the distribution of non-singleton
                                         block lengths, where the length of a block is the
                                         number of base pairs it covers minus 1. That is, a
                                         block with two variants at positions 2 and 5 has
                                         length 3.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "block_n50",
-                    {
-                        "id": "block_n50",
-                        "title": "NG50",
-                        "description": """The NG50 value of the distribution of the block
+                "format": "{:,.0f}",
+                "scale": "Greens",
+                "hidden": False,
+            },
+            "block_n50": {
+                "id": "block_n50",
+                "title": "NG50",
+                "description": """The NG50 value of the distribution of the block
                                         lengths. Interleaved blocks are cut in order to
                                         avoid artificially inflating this value.""",
-                        "format": "{:,.0f}",
-                        "hidden": True,
-                    },
-                ),
-            ]
-        )
+                "format": "{:,.0f}",
+                "scale": "Blues",
+                "hidden": True,
+            },
+        }
 
         general = dict()
         for sample, sample_stats in self.whatshap_stats.items():
@@ -228,8 +266,7 @@ class MultiqcModule(BaseMultiqcModule):
             "tt_percentages": False,
         }
 
-        keys = OrderedDict()
-        keys["Phased Base Pairs"] = {"name": "Phased Base Pairs"}
+        keys = {"Phased Base Pairs": {"name": "Phased Base Pairs"}}
 
         # If the Phased Base Pairs is zero for all samples, we do not add the
         # WhatsHap section
@@ -259,171 +296,130 @@ class MultiqcModule(BaseMultiqcModule):
         # Store the configuration for each column, this is also used to
         # determine which columns to add to the general statistics table
         # https://whatshap.readthedocs.io/en/latest/guide.html#the-tsv-statistics-format
-        stats_headers = OrderedDict(
-            [
-                (
-                    "variants",
-                    {
-                        "id": "variants",
-                        "title": "Input Variants",
-                        "description": """Number of biallelic variants in the input VCF, but
+        stats_headers = {
+            "variants": {
+                "id": "variants",
+                "title": "Input Variants",
+                "description": """Number of biallelic variants in the input VCF, but
                                         excluding any non-SNV variants if --only-snvs was
                                         used""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "heterozygous_variants",
-                    {
-                        "id": "heterozygous_variants",
-                        "title": "Heterozygous Variants",
-                        "description": """The number of biallelic, heterozygous variants in
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "heterozygous_variants": {
+                "id": "heterozygous_variants",
+                "title": "Heterozygous Variants",
+                "description": """The number of biallelic, heterozygous variants in
                                         the input VCF. This is a subset of Input Variants.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "heterozygous_snvs",
-                    {
-                        "id": "heterozygous_snvs",
-                        "title": "Heterozygous SNVs",
-                        "description": """The number of biallelic, heterozygous SNVs in the
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "heterozygous_snvs": {
+                "id": "heterozygous_snvs",
+                "title": "Heterozygous SNVs",
+                "description": """The number of biallelic, heterozygous SNVs in the
                                         input VCF. This is a subset of Heterozygous
                                         Variants.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "unphased",
-                    {
-                        "id": "unphased",
-                        "title": "Unphased Variants",
-                        "description": """The number of biallelic, heterozygous variants that
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "unphased": {
+                "id": "unphased",
+                "title": "Unphased Variants",
+                "description": """The number of biallelic, heterozygous variants that
                                         are not marked as phased in the input VCF. This
                                         is a subset of heterozygous_variants.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "phased",
-                    {
-                        "id": "phased",
-                        "title": "Phased Variants",
-                        "description": """The number of biallelic, heterozygous variants that
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "phased": {
+                "id": "phased",
+                "title": "Phased Variants",
+                "description": """The number of biallelic, heterozygous variants that
                                         are marked as phased in the input VCF. This is
                                         a subset of heterozygous_variants. Also, phased +
                                         unphased + singletons = heterozygous_variants.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "phased_snvs",
-                    {
-                        "id": "phased_snvs",
-                        "title": "Phased SNVs",
-                        "description": """The number of biallelic, heterozygous SNVs that are
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "phased_snvs": {
+                "id": "phased_snvs",
+                "title": "Phased SNVs",
+                "description": """The number of biallelic, heterozygous SNVs that are
                                         marked as phased in the input VCF. This is a subset
                                         of phased.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "blocks",
-                    {
-                        "id": "blocks",
-                        "title": "Blocks",
-                        "description": "The total number of phase sets/blocks.",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "singletons",
-                    {
-                        "id": "singletons",
-                        "title": "Singletons",
-                        "description": "The number of blocks that contain exactly one variant.",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "bp_per_block_sum",
-                    {
-                        "id": "bp_per_block_sum",
-                        "title": "Total Phased bp",
-                        "description": """The sum of the lengths of all non-singleton
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "blocks": {
+                "id": "blocks",
+                "title": "Blocks",
+                "description": "The total number of phase sets/blocks.",
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "singletons": {
+                "id": "singletons",
+                "title": "Singletons",
+                "description": "The number of blocks that contain exactly one variant.",
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "bp_per_block_sum": {
+                "id": "bp_per_block_sum",
+                "title": "Total Phased bp",
+                "description": """The sum of the lengths of all non-singleton
                                         blocks, where the length of a block is the
                                         number of base pairs it covers minus 1. That is, a
                                         block with two variants at positions 2 and 5 has
                                         length 3.""",
-                        "format": "{:,.0f}",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "variant_per_block_avg",
-                    {
-                        "id": "variant_per_block_avg",
-                        "title": "Avg Variants per Block",
-                        "description": """Description of the distribution of non-singleton
+                "format": "{:,.0f}",
+                "hidden": False,
+            },
+            "variant_per_block_avg": {
+                "id": "variant_per_block_avg",
+                "title": "Avg Variants per Block",
+                "description": """Description of the distribution of non-singleton
                                         block sizes, where the size of a block is the number
                                         of variants it contains. Number of biallelic
                                         variants in the input VCF, but excluding any non-SNV
                                         variants if --only-snvs was used.""",
-                        "hidden": False,
-                    },
-                ),
-                (
-                    "frac_het_phased",
-                    {
-                        "id": "perc_het_phased",
-                        "title": "% Phased Variants",
-                        "description": """Fraction of heterozygous variants
+                "hidden": False,
+            },
+            "frac_het_phased": {
+                "id": "perc_het_phased",
+                "title": "% Phased Variants",
+                "description": """Fraction of heterozygous variants
                                           that could be phased.
                                         """,
-                        "format": "{:,.0f}",
-                        "suffix": "%",
-                        "min": 0,
-                        "max": 100,
-                        "modify": lambda x: x * 100,
-                        "hidden": True,
-                    },
-                ),
-                (
-                    "bp_per_block_avg",
-                    {
-                        "id": "bp_per_block_avg",
-                        "title": "Avg bp per Block",
-                        "description": """Description of the distribution of non-singleton
+                "format": "{:,.0f}",
+                "suffix": "%",
+                "min": 0,
+                "max": 100,
+                "modify": lambda x: x * 100,
+                "hidden": True,
+            },
+            "bp_per_block_avg": {
+                "id": "bp_per_block_avg",
+                "title": "Avg bp per Block",
+                "description": """Description of the distribution of non-singleton
                                         block lengths, where the length of a block is the
                                         number of base pairs it covers minus 1. That is, a
                                         block with two variants at positions 2 and 5 has
                                         length 3.""",
-                        "format": "{:,.0f}",
-                        "hidden": True,
-                    },
-                ),
-                (
-                    "block_n50",
-                    {
-                        "id": "block_n50",
-                        "title": "NG50",
-                        "description": """The NG50 value of the distribution of the block
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+            "block_n50": {
+                "id": "block_n50",
+                "title": "NG50",
+                "description": """The NG50 value of the distribution of the block
                                         lengths. Interleaved blocks are cut in order to
                                         avoid artificially inflating this value.""",
-                        "format": "{:,.0f}",
-                        "hidden": True,
-                    },
-                ),
-            ]
-        )
+                "format": "{:,.0f}",
+                "hidden": True,
+            },
+        }
 
         general = dict()
         for sample, sample_stats in self.whatshap_stats.items():
@@ -437,4 +433,8 @@ class MultiqcModule(BaseMultiqcModule):
 
             general[sample] = stats
 
-        self.add_section(name="WhatsHap statistics", anchor="whatshap-table", plot=table.plot(general, stats_headers))
+        self.add_section(
+            name="WhatsHap statistics",
+            anchor="whatshap-table",
+            plot=table.plot(general, stats_headers),
+        )
