@@ -12,9 +12,8 @@ import os
 import random
 import re
 import sys
-from collections import OrderedDict
 
-from multiqc.utils import config, report, util_functions
+from multiqc.utils import config, mqc_colour, report, util_functions
 
 logger = logging.getLogger(__name__)
 
@@ -49,11 +48,11 @@ def get_template_mod():
 
 def plot(data, cats=None, pconfig=None):
     """Plot a horizontal bar graph. Expects a 2D dict of sample
-    data. Also can take info about categories. There are quite a
+    data. Also, can take info about categories. There are quite a
     few variants of how to use this function, see the docs for details.
     :param data: 2D dict, first keys as sample names, then x:y data pairs
                  Can supply a list of dicts and will have buttons to switch
-    :param cats: optional list, dict or OrderedDict with plot categories
+    :param cats: optional list or dict with plot categories
     :param pconfig: optional dict with config key:value pairs
     :return: HTML and JS, ready to be inserted into the page
     """
@@ -67,7 +66,7 @@ def plot(data, cats=None, pconfig=None):
             pconfig[k] = v
 
     # Validate config if linting
-    if config.lint:
+    if config.strict:
         # Get module name
         modname = ""
         callstack = inspect.stack()
@@ -91,28 +90,21 @@ def plot(data, cats=None, pconfig=None):
             report.lint_errors.append(errmsg)
 
     # Given one dataset - turn it into a list
-    if type(data) is not list:
+    if not isinstance(data, list):
         data = [data]
 
     # Make list of cats from different inputs
     if cats is None:
         cats = list()
-    elif type(cats) is not list:
+    elif not isinstance(cats, list):
+        cats = [cats]
+    elif cats and isinstance(cats[0], str):
         cats = [cats]
-    else:
-        try:  # Py2
-            if type(cats[0]) is str or type(cats[0]) is unicode:
-                cats = [cats]
-        except NameError:  # Py3
-            if type(cats[0]) is str:
-                cats = [cats]
-        except IndexError:  # Given empty list
-            pass
     # Generate default categories if not supplied
     for idx in range(len(data)):
         try:
             cats[idx]
-        except (IndexError):
+        except IndexError:
             cats.append(list())
             for s in data[idx].keys():
                 for k in data[idx][s].keys():
@@ -121,8 +113,8 @@ def plot(data, cats=None, pconfig=None):
 
     # If we have cats in lists, turn them into dicts
     for idx, cat in enumerate(cats):
-        if type(cat) is list:
-            newcats = OrderedDict()
+        if isinstance(cat, list):
+            newcats = dict()
             for c in cat:
                 newcats[c] = {"name": c}
             cats[idx] = newcats
@@ -134,18 +126,16 @@ def plot(data, cats=None, pconfig=None):
     # Allow user to overwrite a given category config for this plot
     if "id" in pconfig and pconfig["id"] and pconfig["id"] in config.custom_plot_config:
         for k, v in config.custom_plot_config[pconfig["id"]].items():
-            if k in cats[idx].keys():
-                for kk, vv in v.items():
-                    cats[idx][k][kk] = vv
+            for idx in range(len(cats)):
+                if k in cats[idx].keys():
+                    for kk, vv in v.items():
+                        cats[idx][k][kk] = vv
 
     # Parse the data into a chart friendly format
     plotsamples = list()
     plotdata = list()
     for idx, d in enumerate(data):
-        if isinstance(d, OrderedDict):
-            hc_samples = list(d.keys())
-        else:
-            hc_samples = sorted(list(d.keys()))
+        hc_samples = sorted(list(d.keys()))
         hc_data = list()
         sample_dcount = dict()
         for c in cats[idx].keys():
@@ -183,30 +173,46 @@ def plot(data, cats=None, pconfig=None):
         logger.warning(f"Tried to make bar plot, but had no data: {pconfig.get('id')}")
         return '<p class="text-danger">Error - was not able to plot data.</p>'
 
+    # Add colors to the categories if not set. Since the "plot_defaults" scale is
+    # identical to default scale of the Highcharts JS library, this is not strictly
+    # needed. But it future proofs when we replace Highcharts with something else.
+    scale = mqc_colour.mqc_colour_scale("plot_defaults")
+    for si, sd in enumerate(plotdata):
+        for di, d in enumerate(sd):
+            d.setdefault("color", scale.get_colour(di, lighten=1))
+
     # Make a plot - custom, interactive or flat
-    try:
-        return get_template_mod().bargraph(plotdata, plotsamples, pconfig)
-    except (AttributeError, TypeError):
-        if config.plots_force_flat or (
-            not config.plots_force_interactive and len(plotsamples[0]) > config.plots_flat_numseries
-        ):
-            try:
-                report.num_mpl_plots += 1
-                return matplotlib_bargraph(plotdata, plotsamples, pconfig)
-            except Exception as e:
-                logger.error("############### Error making MatPlotLib figure! Falling back to HighCharts.")
-                logger.debug(e, exc_info=True)
-                return highcharts_bargraph(plotdata, plotsamples, pconfig)
-        else:
-            # Use MatPlotLib to generate static plots if requested
-            if config.export_plots:
-                try:
-                    matplotlib_bargraph(plotdata, plotsamples, pconfig)
-                except Exception as e:
-                    logger.error("############### Error making MatPlotLib figure! Plot not exported.")
-                    logger.debug(e, exc_info=True)
-            # Return HTML for HighCharts dynamic plot
+    mod = get_template_mod()
+    if "bargraph" in mod.__dict__ and callable(mod.bargraph):
+        try:
+            return mod.bargraph(plotdata, plotsamples, pconfig)
+        except:  # noqa: E722
+            if config.strict:
+                # Crash quickly in the strict mode. This can be helpful for interactive
+                # debugging of modules
+                raise
+    if config.plots_force_flat or (
+        not config.plots_force_interactive and len(plotsamples[0]) > config.plots_flat_numseries
+    ):
+        try:
+            report.num_mpl_plots += 1
+            return matplotlib_bargraph(plotdata, plotsamples, pconfig)
+        except Exception as e:
+            if config.strict:
+                raise
+            logger.error("############### Error making MatPlotLib figure! Falling back to HighCharts.")
+            logger.debug(e, exc_info=True)
             return highcharts_bargraph(plotdata, plotsamples, pconfig)
+    else:
+        # Use MatPlotLib to generate static plots if requested
+        if config.export_plots:
+            try:
+                matplotlib_bargraph(plotdata, plotsamples, pconfig)
+            except Exception as e:
+                logger.error("############### Error making MatPlotLib figure! Plot not exported.")
+                logger.debug(e, exc_info=True)
+        # Return HTML for HighCharts dynamic plot
+        return highcharts_bargraph(plotdata, plotsamples, pconfig)
 
 
 def highcharts_bargraph(plotdata, plotsamples=None, pconfig=None):
@@ -266,18 +272,18 @@ def highcharts_bargraph(plotdata, plotsamples=None, pconfig=None):
             active = "active" if k == 0 else ""
             try:
                 name = pconfig["data_labels"][k]["name"]
-            except:
+            except Exception:
                 try:
                     name = pconfig["data_labels"][k]
-                except:
+                except Exception:
                     name = k + 1
             try:
                 ylab = 'data-ylab="{}"'.format(pconfig["data_labels"][k]["ylab"])
-            except:
+            except Exception:
                 ylab = 'data-ylab="{}"'.format(name) if name != k + 1 else ""
             try:
                 ymax = 'data-ymax="{}"'.format(pconfig["data_labels"][k]["ymax"])
-            except:
+            except Exception:
                 ymax = ""
             html += '<button class="btn btn-default btn-sm {a}" data-action="set_data" {y} {ym} data-newdata="{k}" data-target="{id}">{n}</button>\n'.format(
                 a=active, id=pconfig["id"], n=name, y=ylab, ym=ymax, k=k
@@ -326,7 +332,7 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
     for k in range(len(plotdata)):
         try:
             name = pconfig["data_labels"][k]
-        except:
+        except Exception:
             name = k + 1
         pid = "mqc_{}_{}".format(pconfig["id"], name)
         pid = report.save_htmlid(pid, skiplint=True)
@@ -339,20 +345,6 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
     )
     html += '<div class="mqc_mplplot_plotgroup" id="{}">'.format(pconfig["id"])
 
-    # Same defaults as HighCharts for consistency
-    default_colors = [
-        "#7cb5ec",
-        "#434348",
-        "#90ed7d",
-        "#f7a35c",
-        "#8085e9",
-        "#f15c80",
-        "#e4d354",
-        "#2b908f",
-        "#f45b5b",
-        "#91e8e1",
-    ]
-
     # Counts / Percentages Switch
     if pconfig.get("cpswitch") is not False and not config.simple_output:
         if pconfig.get("cpswitch_c_active", True) is True:
@@ -367,9 +359,7 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
         html += '<div class="btn-group mpl_switch_group mqc_mplplot_bargraph_setcountspcnt"> \n\
             <button class="btn btn-default btn-sm {c_a} counts">{c_l}</button> \n\
             <button class="btn btn-default btn-sm {p_a} pcnt">{p_l}</button> \n\
-        </div> '.format(
-            c_a=c_active, p_a=p_active, c_l=c_label, p_l=p_label
-        )
+        </div> '.format(c_a=c_active, p_a=p_active, c_l=c_label, p_l=p_label)
         if len(plotdata) > 1:
             html += " &nbsp; &nbsp; "
 
@@ -381,7 +371,7 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
             active = "active" if k == 0 else ""
             try:
                 name = pconfig["data_labels"][k]
-            except:
+            except Exception:
                 name = k + 1
             html += '<button class="btn btn-default btn-sm {a}" data-target="#{pid}">{n}</button>\n'.format(
                 a=active, pid=pid, n=name
@@ -390,7 +380,6 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
 
     # Go through datasets creating plots
     for pidx, pdata in enumerate(plotdata):
-
         # Save plot data to file
         fdata = {}
         for d in pdata:
@@ -412,7 +401,6 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
             pdata[idx]["data"] = [x if not math.isnan(x) else 0 for x in d["data"]]
 
         for plot_pct in plot_pcts:
-
             # Plot ID
             pid = pids[pidx]
             hide_plot = False
@@ -459,7 +447,7 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
                 if len(values) < len(y_ind):
                     values.extend([0] * (len(y_ind) - len(values)))
                 if plot_pct is True:
-                    for (key, var) in enumerate(values):
+                    for key, var in enumerate(values):
                         s_total = s_totals[key]
                         if s_total == 0:
                             values[key] = 0
@@ -472,10 +460,6 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
                 else:
                     for i, p in enumerate(prevdata):
                         prevdata[i] += prev_values[i]
-                # Default colour index
-                cidx = idx
-                while cidx >= len(default_colors):
-                    cidx -= len(default_colors)
                 # Save the name of this series
                 dlabels.append(d["name"])
                 # Add the series of bars to the plot
@@ -484,7 +468,7 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
                     values,
                     bar_width,
                     left=prevdata,
-                    color=d.get("color", default_colors[cidx]),
+                    color=d["color"],
                     align="center",
                     linewidth=pconfig.get("borderWidth", 0),
                 )
@@ -519,7 +503,7 @@ def matplotlib_bargraph(plotdata, plotsamples, pconfig=None):
             axes.spines["top"].set_visible(False)
             axes.spines["bottom"].set_visible(False)
             axes.spines["left"].set_visible(False)
-            plt.gca().invert_yaxis()  # y axis is reverse sorted otherwise
+            plt.gca().invert_yaxis()  # y-axis is reverse sorted otherwise
 
             # Hide some labels if we have a lot of samples
             show_nth = max(1, math.ceil(len(pdata[0]["data"]) / 150))
