@@ -2,8 +2,6 @@
 
 """ MultiQC functions to plot a linegraph """
 
-from __future__ import print_function, division
-from collections import OrderedDict
 import base64
 import inspect
 import io
@@ -13,7 +11,7 @@ import random
 import re
 import sys
 
-from multiqc.utils import config, report, util_functions
+from multiqc.utils import config, mqc_colour, report, util_functions
 
 logger = logging.getLogger(__name__)
 
@@ -63,11 +61,11 @@ def plot(data, pconfig=None):
             pconfig[k] = v
 
     # Given one dataset - turn it into a list
-    if type(data) is not list:
+    if not isinstance(data, list):
         data = [data]
 
     # Validate config if linting
-    if config.lint:
+    if config.strict:
         # Get module name
         modname = ""
         callstack = inspect.stack()
@@ -94,7 +92,7 @@ def plot(data, pconfig=None):
     if pconfig.get("smooth_points", None) is not None:
         sumcounts = pconfig.get("smooth_points_sumcounts", True)
         for i, d in enumerate(data):
-            if type(sumcounts) is list:
+            if isinstance(sumcounts, list):
                 sumc = sumcounts[i]
             else:
                 sumc = sumcounts
@@ -122,18 +120,17 @@ def plot(data, pconfig=None):
         thisplotdata = list()
 
         for s in sorted(d.keys()):
-
             # Ensure any overwritten conditionals from data_labels (e.g. ymax) are taken in consideration
             series_config = pconfig.copy()
-            if (
-                "data_labels" in pconfig and type(pconfig["data_labels"][data_index]) is dict
+            if "data_labels" in pconfig and isinstance(
+                pconfig["data_labels"][data_index], dict
             ):  # if not a dict: only dataset name is provided
                 series_config.update(pconfig["data_labels"][data_index])
 
             pairs = list()
             maxval = 0
             if "categories" in series_config:
-                if "categories" not in pconfig or type(pconfig["categories"]) is not list:
+                if "categories" not in pconfig or not isinstance(pconfig["categories"], list):
                     pconfig["categories"] = list()
                 # Add any new categories
                 for k in d[s].keys():
@@ -147,7 +144,6 @@ def plot(data, pconfig=None):
                     except KeyError:
                         pairs.append(None)
             else:
-
                 # Discard > ymax or just hide?
                 # If it never comes back into the plot, discard. If it goes above then comes back, just hide.
                 discard_ymax = None
@@ -197,7 +193,7 @@ def plot(data, pconfig=None):
                 this_series = {"name": s, "data": pairs}
                 try:
                     this_series["color"] = series_config["colors"][s]
-                except:
+                except Exception:
                     pass
                 thisplotdata.append(this_series)
         plotdata.append(thisplotdata)
@@ -206,9 +202,9 @@ def plot(data, pconfig=None):
     try:
         if pconfig.get("extra_series"):
             extra_series = pconfig["extra_series"]
-            if type(pconfig["extra_series"]) == dict:
+            if isinstance(pconfig["extra_series"], dict):
                 extra_series = [[pconfig["extra_series"]]]
-            elif type(pconfig["extra_series"]) == list and type(pconfig["extra_series"][0]) == dict:
+            elif isinstance(pconfig["extra_series"], list) and isinstance(pconfig["extra_series"][0], dict):
                 extra_series = [pconfig["extra_series"]]
             for i, es in enumerate(extra_series):
                 for s in es:
@@ -216,26 +212,42 @@ def plot(data, pconfig=None):
     except (KeyError, IndexError):
         pass
 
+    # Add colors to the categories if not set. Since the "plot_defaults" scale is
+    # identical to default scale of the Highcharts JS library, this is not strictly
+    # needed. But it future proofs when we replace Highcharts with something else.
+    scale = mqc_colour.mqc_colour_scale("plot_defaults")
+    for si, sd in enumerate(plotdata):
+        for di, d in enumerate(sd):
+            d.setdefault("color", scale.get_colour(di, lighten=1))
+
     # Make a plot - template custom, or interactive or flat
-    try:
-        return get_template_mod().linegraph(plotdata, pconfig)
-    except (AttributeError, TypeError):
-        if config.plots_force_flat or (
-            not config.plots_force_interactive and plotdata and len(plotdata[0]) > config.plots_flat_numseries
-        ):
-            try:
-                report.num_mpl_plots += 1
-                return matplotlib_linegraph(plotdata, pconfig)
-            except Exception as e:
-                logger.error("############### Error making MatPlotLib figure! Falling back to HighCharts.")
-                logger.debug(e, exc_info=True)
-                return highcharts_linegraph(plotdata, pconfig)
-        else:
-            # Use MatPlotLib to generate static plots if requested
-            if config.export_plots:
-                matplotlib_linegraph(plotdata, pconfig)
-            # Return HTML for HighCharts dynamic plot
+    mod = get_template_mod()
+    if "linegraph" in mod.__dict__ and callable(mod.linegraph):
+        try:
+            return mod.linegraph(plotdata, pconfig)
+        except:  # noqa: E722
+            if config.strict:
+                # Crash quickly in the strict mode. This can be helpful for interactive
+                # debugging of modules
+                raise
+    if config.plots_force_flat or (
+        not config.plots_force_interactive and plotdata and len(plotdata[0]) > config.plots_flat_numseries
+    ):
+        try:
+            report.num_mpl_plots += 1
+            return matplotlib_linegraph(plotdata, pconfig)
+        except Exception as e:
+            if config.strict:
+                raise
+            logger.error("############### Error making MatPlotLib figure! Falling back to HighCharts.")
+            logger.debug(e, exc_info=True)
             return highcharts_linegraph(plotdata, pconfig)
+    else:
+        # Use MatPlotLib to generate static plots if requested
+        if config.export_plots:
+            matplotlib_linegraph(plotdata, pconfig)
+        # Return HTML for HighCharts dynamic plot
+        return highcharts_linegraph(plotdata, pconfig)
 
 
 def highcharts_linegraph(plotdata, pconfig=None):
@@ -284,19 +296,19 @@ def highcharts_linegraph(plotdata, pconfig=None):
             active = "active" if k == 0 else ""
             try:
                 name = pconfig["data_labels"][k]["name"]
-            except:
+            except Exception:
                 name = k + 1
             try:
                 ylab = 'data-ylab="{}"'.format(pconfig["data_labels"][k]["ylab"])
-            except:
+            except Exception:
                 ylab = 'data-ylab="{}"'.format(name) if name != k + 1 else ""
             try:
                 ymax = 'data-ymax="{}"'.format(pconfig["data_labels"][k]["ymax"])
-            except:
+            except Exception:
                 ymax = ""
             try:
                 xlab = 'data-xlab="{}"'.format(pconfig["data_labels"][k]["xlab"])
-            except:
+            except Exception:
                 xlab = ""
             html += '<button class="btn btn-default btn-sm {a}" data-action="set_data" {y} {ym} {x} data-newdata="{k}" data-target="{id}">{n}</button>\n'.format(
                 a=active, id=pconfig["id"], n=name, y=ylab, ym=ymax, x=xlab, k=k
@@ -337,7 +349,7 @@ def matplotlib_linegraph(plotdata, pconfig=None):
     for k in range(len(plotdata)):
         try:
             name = pconfig["data_labels"][k]["name"]
-        except:
+        except Exception:
             name = k + 1
         pid = "mqc_{}_{}".format(pconfig["id"], name)
         pid = report.save_htmlid(pid, skiplint=True)
@@ -350,20 +362,6 @@ def matplotlib_linegraph(plotdata, pconfig=None):
     )
     html += '<div class="mqc_mplplot_plotgroup" id="{}">'.format(pconfig["id"])
 
-    # Same defaults as HighCharts for consistency
-    default_colors = [
-        "#7cb5ec",
-        "#434348",
-        "#90ed7d",
-        "#f7a35c",
-        "#8085e9",
-        "#f15c80",
-        "#e4d354",
-        "#2b908f",
-        "#f45b5b",
-        "#91e8e1",
-    ]
-
     # Buttons to cycle through different datasets
     if len(plotdata) > 1 and not config.simple_output:
         html += '<div class="btn-group mpl_switch_group mqc_mplplot_bargraph_switchds">\n'
@@ -372,7 +370,7 @@ def matplotlib_linegraph(plotdata, pconfig=None):
             active = "active" if k == 0 else ""
             try:
                 name = pconfig["data_labels"][k]["name"]
-            except:
+            except Exception:
                 name = k + 1
             html += '<button class="btn btn-default btn-sm {a}" data-target="#{pid}">{n}</button>\n'.format(
                 a=active, pid=pid, n=name
@@ -381,32 +379,34 @@ def matplotlib_linegraph(plotdata, pconfig=None):
 
     # Go through datasets creating plots
     for pidx, pdata in enumerate(plotdata):
-
         # Plot ID
         pid = pids[pidx]
 
         # Save plot data to file
         if pconfig.get("save_data_file", True):
-            fdata = OrderedDict()
+            fdata = dict()
             lastcats = None
             sharedcats = True
             for d in pdata:
-                fdata[d["name"]] = OrderedDict()
+                fdata[d["name"]] = dict()
+
+                # Check to see if all categories are the same
+                if len(d["data"]) > 0 and isinstance(d["data"][0], list):
+                    if lastcats is None:
+                        lastcats = [x[0] for x in d["data"]]
+                    elif lastcats != [x[0] for x in d["data"]]:
+                        sharedcats = False
+
                 for i, x in enumerate(d["data"]):
-                    if type(x) is list:
-                        fdata[d["name"]][str(x[0])] = x[1]
-                        # Check to see if all categories are the same
-                        if lastcats is None:
-                            lastcats = [x[0] for x in d["data"]]
-                        elif lastcats != [x[0] for x in d["data"]]:
-                            sharedcats = False
+                    if isinstance(x, list):
+                        fdata[d["name"]][x[0]] = x[1]
                     else:
                         try:
                             fdata[d["name"]][pconfig["categories"][i]] = x
                         except (KeyError, IndexError):
                             fdata[d["name"]][str(i)] = x
 
-            # Custom tsv output if the x axis varies
+            # Custom tsv output if the x-axis varies
             if not sharedcats and config.data_format == "tsv":
                 fout = ""
                 for d in pdata:
@@ -432,12 +432,6 @@ def matplotlib_linegraph(plotdata, pconfig=None):
 
         # Go through data series
         for idx, d in enumerate(pdata):
-
-            # Default colour index
-            cidx = idx
-            while cidx >= len(default_colors):
-                cidx -= len(default_colors)
-
             # Line style
             linestyle = "solid"
             if d.get("dashStyle", None) == "Dash":
@@ -449,16 +443,20 @@ def matplotlib_linegraph(plotdata, pconfig=None):
                     [x[0] for x in d["data"]],
                     [x[1] for x in d["data"]],
                     label=d["name"],
-                    color=d.get("color", default_colors[cidx]),
+                    color=d["color"],
                     linestyle=linestyle,
                     linewidth=1,
                     marker=None,
                 )
             except TypeError:
                 # Categorical data on x axis
-                axes.plot(
-                    d["data"], label=d["name"], color=d.get("color", default_colors[cidx]), linewidth=1, marker=None
-                )
+                axes.plot(d["data"], label=d["name"], color=d["color"], linewidth=1, marker=None)
+
+        # Log scale
+        if pconfig.get("xLog", False):
+            axes.set_xscale("log")
+        if pconfig.get("yLog", False):
+            axes.set_yscale("log")
 
         # Tidy up axes
         axes.tick_params(
@@ -470,7 +468,7 @@ def matplotlib_linegraph(plotdata, pconfig=None):
         # Dataset specific y label
         try:
             axes.set_ylabel(pconfig["data_labels"][pidx]["ylab"])
-        except:
+        except Exception:
             pass
 
         # Axis limits
@@ -492,7 +490,7 @@ def matplotlib_linegraph(plotdata, pconfig=None):
         # Dataset specific ymax
         try:
             axes.set_ylim((ymin, pconfig["data_labels"][pidx]["ymax"]))
-        except:
+        except Exception:
             pass
 
         default_xlimits = axes.get_xlim()
@@ -507,7 +505,7 @@ def matplotlib_linegraph(plotdata, pconfig=None):
         elif "xCeiling" in pconfig:
             xmax = min(pconfig["xCeiling"], default_xlimits[1])
         if (xmax - xmin) < pconfig.get("xMinRange", 0):
-            xmax = xmin + pconfig["xMinRange"]
+            xmax = xmin + pconfig.get("xMinRange", 0)
         axes.set_xlim((xmin, xmax))
 
         # Plot title
@@ -643,7 +641,7 @@ def smooth_line_data(data, numpoints, sumcounts=True):
 
         binsize = (len(d) - 1) / (numpoints - 1)
         first_element_indices = [round(binsize * i) for i in range(numpoints)]
-        smoothed_d = OrderedDict(xy for i, xy in enumerate(d.items()) if i in first_element_indices)
+        smoothed_d = {x: y for i, (x, y) in enumerate(d.items()) if i in first_element_indices}
         smoothed_data[s_name] = smoothed_d
 
     return smoothed_data
