@@ -13,26 +13,22 @@ import mimetypes
 import os
 import re
 import time
-from collections import OrderedDict, defaultdict
+from collections import defaultdict, OrderedDict
 
-import lzstring
 import rich
 import rich.progress
 import yaml
+from yaml.representer import Representer
+
+from multiqc.utils import lzstring
 
 from . import config
 
 logger = config.logger
 
 # Treat defaultdict and OrderedDict as normal dicts for YAML output
-from yaml.representer import Representer, SafeRepresenter
-
 yaml.add_representer(defaultdict, Representer.represent_dict)
 yaml.add_representer(OrderedDict, Representer.represent_dict)
-try:
-    yaml.add_representer(unicode, SafeRepresenter.represent_unicode)
-except NameError:
-    pass  # Python 3
 
 
 # Set up global variables shared across modules
@@ -100,6 +96,10 @@ def init():
     global files
     files = dict()
 
+    # Map of software tools to a set of unique version strings
+    global software_versions
+    software_versions = defaultdict(lambda: defaultdict(list))
+
 
 def get_filelist(run_module_names):
     """
@@ -197,6 +197,14 @@ def get_filelist(run_module_names):
                 file_search_stats["skipped_filesize_limit"] += 1
                 return False
 
+        # Use mimetypes to exclude binary files where possible
+        if not re.match(r".+_mqc\.(png|jpg|jpeg)", f["fn"]) and config.ignore_images:
+            (ftype, encoding) = mimetypes.guess_type(os.path.join(f["root"], f["fn"]))
+            if encoding is not None:
+                return False
+            if ftype is not None and ftype.startswith("image"):
+                return False
+
         # Test file for each search pattern
         file_matched = False
         for patterns in spatterns:
@@ -313,6 +321,8 @@ def get_filelist(run_module_names):
         progress.update(mqc_task, s_fn="")
 
     runtimes["total_sp"] = time.time() - total_sp_starttime
+    if config.profile_runtime:
+        logger.info(f"Profile-runtime: Searching files took {runtimes['total_sp']:.2f}s")
 
     # Debug log summary about what we skipped
     summaries = []
@@ -331,14 +341,6 @@ def search_file(pattern, f, module_key):
     fn_matched = False
     contents_matched = False
 
-    # Use mimetypes to exclude binary files where possible
-    if not re.match(r".+_mqc\.(png|jpg|jpeg)", f["fn"]) and config.ignore_images:
-        (ftype, encoding) = mimetypes.guess_type(os.path.join(f["root"], f["fn"]))
-        if encoding is not None:
-            return False
-        if ftype is not None and ftype.startswith("image"):
-            return False
-
     # Search pattern specific filesize limit
     if pattern.get("max_filesize") is not None and "filesize" in f:
         if f["filesize"] > pattern.get("max_filesize"):
@@ -351,6 +353,8 @@ def search_file(pattern, f, module_key):
             fn_matched = True
             if pattern.get("contents") is None and pattern.get("contents_re") is None:
                 return True
+        else:
+            return False
 
     # Search by file name (regex)
     if pattern.get("fn_re") is not None:
@@ -358,40 +362,49 @@ def search_file(pattern, f, module_key):
             fn_matched = True
             if pattern.get("contents") is None and pattern.get("contents_re") is None:
                 return True
+        else:
+            return False
 
     # Search by file contents
     if pattern.get("contents") is not None or pattern.get("contents_re") is not None:
         if pattern.get("contents_re") is not None:
             repattern = re.compile(pattern["contents_re"])
-        try:
+        if "contents_lines" not in f or ("num_lines" in pattern and len(f["contents_lines"]) < pattern["num_lines"]):
+            f["contents_lines"] = []
             file_path = os.path.join(f["root"], f["fn"])
-            with io.open(file_path, "r", encoding="utf-8") as fh:
-                l = 1
-                for line in fh:
-                    # Search by file contents (string)
-                    if pattern.get("contents") is not None:
-                        if pattern["contents"] in line:
-                            contents_matched = True
-                            if pattern.get("fn") is None and pattern.get("fn_re") is None:
-                                return True
+            try:
+                with io.open(file_path, "r", encoding="utf-8") as fh:
+                    for i, line in enumerate(fh):
+                        f["contents_lines"].append(line)
+                        if i >= config.filesearch_lines_limit and i >= pattern.get("num_lines", 0):
                             break
-                    # Search by file contents (regex)
-                    elif pattern.get("contents_re") is not None:
-                        if re.search(repattern, line):
-                            contents_matched = True
-                            if pattern.get("fn") is None and pattern.get("fn_re") is None:
-                                return True
-                            break
-                    # Break if we've searched enough lines for this pattern
-                    if pattern.get("num_lines") and l >= pattern.get("num_lines"):
-                        break
-                    l += 1
-        # Can't open file - usually because it's a binary file and we're reading as utf-8
-        except (IOError, OSError, ValueError, UnicodeDecodeError) as e:
-            if config.report_readerrors:
-                logger.debug(f"Couldn't read file when looking for output: {file_path}, {e}")
-            file_search_stats["skipped_file_contents_search_errors"] += 1
-            return False
+            # Can't open file - usually because it's a binary file, and we're reading as utf-8
+            except (IOError, OSError, ValueError, UnicodeDecodeError) as e:
+                if config.report_readerrors:
+                    logger.debug(f"Couldn't read file when looking for output: {file_path}, {e}")
+                file_search_stats["skipped_file_contents_search_errors"] += 1
+                return False
+
+        # Go through the parsed file contents
+        for i, line in enumerate(f["contents_lines"]):
+            # Break if we've searched enough lines for this pattern
+            if pattern.get("num_lines") and i >= pattern.get("num_lines"):
+                break
+
+            # Search by file contents (string)
+            if pattern.get("contents") is not None:
+                if pattern["contents"] in line:
+                    contents_matched = True
+                    if pattern.get("fn") is None and pattern.get("fn_re") is None:
+                        return True
+                    break
+            # Search by file contents (regex)
+            elif pattern.get("contents_re") is not None:
+                if re.search(repattern, line):
+                    contents_matched = True
+                    if pattern.get("fn") is None and pattern.get("fn_re") is None:
+                        return True
+                    break
 
     return fn_matched and contents_matched
 
@@ -451,16 +464,16 @@ def data_sources_tofile():
                 for sec in data_sources[mod]:
                     for s_name, source in data_sources[mod][sec].items():
                         lines.append([mod, sec, s_name, source])
-            body = "\n".join(["\t".join(l) for l in lines])
+            body = "\n".join(["\t".join(line) for line in lines])
             print(body.encode("utf-8", "ignore").decode("utf-8"), file=f)
 
 
-def dois_tofile():
+def dois_tofile(modules_output):
     """Find all DOIs listed in report sections and write to a file"""
     # Collect DOIs
     dois = {"MultiQC": ["10.1093/bioinformatics/btw354"]}
     for mod in modules_output:
-        if mod.doi is not None and mod.doi != []:
+        if mod.doi is not None and mod.doi != "" and mod.doi != []:
             dois[mod.anchor] = mod.doi
     # Write to a file
     fn = "multiqc_citations.{}".format(config.data_format_extensions[config.data_format])
@@ -498,7 +511,7 @@ def save_htmlid(html_id, skiplint=False):
     html_id_clean = re.sub("[^a-zA-Z0-9_-]+", "_", html_id_clean)
 
     # Validate if linting
-    if config.lint and not skiplint:
+    if config.strict and not skiplint:
         modname = ""
         codeline = ""
         callstack = inspect.stack()
@@ -508,7 +521,7 @@ def save_htmlid(html_id, skiplint=False):
                 modname = ">{}< ".format(callpath)
                 codeline = n[4][0].strip()
                 break
-    if config.lint and not skiplint and html_id != html_id_clean:
+    if config.strict and not skiplint and html_id != html_id_clean:
         errmsg = "LINT: {}HTML ID was not clean ('{}' -> '{}') ## {}".format(modname, html_id, html_id_clean, codeline)
         logger.error(errmsg)
         lint_errors.append(errmsg)
@@ -519,7 +532,7 @@ def save_htmlid(html_id, skiplint=False):
     while html_id_clean in html_ids:
         html_id_clean = "{}-{}".format(html_id_base, i)
         i += 1
-        if config.lint and not skiplint:
+        if config.strict and not skiplint:
             errmsg = "LINT: {}HTML ID was a duplicate ({}) ## {}".format(modname, html_id_clean, codeline)
             logger.error(errmsg)
             lint_errors.append(errmsg)
